#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: ttttmr
@Date: 2019-05-01 12:07:54
@LastEditTime: 2019-06-26 00:25:14
'''

import hashlib
import random
import sys
import urllib

import requests

from ..core.common import outputscreen
from ..core.data import conf

USER_AGENT = "Mozilla/5.0 (Windows; U; MSIE 10.0; Windows NT 9.0; es-ES)"
user_agent = {"user-agent": USER_AGENT}


class Inspector:
    """This class mission is to examine the behaviour of the application when on
        purpose an inexistent page is requested"""
    TEST404_OK = 0
    TEST404_MD5 = 1
    TEST404_STRING = 2
    TEST404_URL = 3
    TEST404_NONE = 4

    def __init__(self, target):
        self.target = target

    def _give_it_a_try(self):
        """Every time this method is called it will request a random resource
            the target domain. Return value is a dictionary with values as
            HTTP response code, resquest size, md5 of the content and the content
            itself. If there were a redirection it will record the new url"""
        s = []
        for n in range(0, 42):
            random.seed()
            s.append(chr(random.randrange(97, 122)))
        s = "".join(s)
        target = self.target + s

        outputscreen.success(f"[+] Checking with: {target}")

        try:
            page = requests.get(target, headers=user_agent, verify=False,timeout=5, proxies=conf.proxy_server)
            content = page.content
            result = {
                    'target': urllib.parse.urlparse(target).netloc,
                    'code': str(page.status_code),
                    'size': len(content),
                    'md5': hashlib.md5(content).hexdigest(),
                    'content': content,
                    'location': None
                }

            if len(page.history) >= 1:
                result['location'] = page.url
            return result
        except:
            result = {
                    'target': urllib.parse.urlparse(target).netloc,
                    'code': '',
                    'size': '',
                    'md5': '',
                    'content': '',
                    'location': None
                }
            return result

    def check_this(self):
        """Get the a request and decide what to do"""
        first_result = self._give_it_a_try()

        if first_result['code'] == '404':
            #msg = '[+] Target: {} Got a nice 404, problems not expected'
            #outputscreen.success("\r{}{}".format(self.target,' '*(th.console_width-len(msg)+len(self.target)+1)))
            # Ok, resquest gave a 404 so we should not find problems
            return '', Inspector.TEST404_OK

        elif first_result['code'] == '302' or first_result['location']:
            location = first_result['location']
            return location, Inspector.TEST404_URL
        else:
            return first_result['md5'], Inspector.TEST404_MD5

        # We give up here :(
        return '', Inspector.TEST404_NONE

if __name__ == '__main__':
    i = Inspector(sys.argv[1])
    print(i.check_this())



#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: xxlin
@Date: 2019-03-14 09:49:05
@LastEditTime: 2023-07-25 16:32:33
'''

import hashlib
import os
import random
import re
import sys
import time
import urllib

import gevent
import progressbar
import requests
from gevent.queue import Queue
from lxml import etree

from ..core.common import intToSize, outputscreen, urlSimilarCheck
from ..core.data import bar, conf, paths, payloads, tasks, th
from ..utils.config import ConfigFileParser
from ..plugins.inspector import Inspector

#防止ssl未校验时出现提示信息
requests.packages.urllib3.disable_warnings()

#dict_mode的payloads
payloads.dict_mode_dict = set()
#crawl_mode的payloads
payloads.crawl_mode_dynamic_fuzz_temp_dict = set()
payloads.similar_urls_set = set()
payloads.crawl_mode_dynamic_fuzz_dict = list()
#blast_mode的payload
payloads.blast_mode_custom_charset_dict = list()
#fuzz_mode的payload
payloads.fuzz_mode_dict = list()

#创建all_tasks队列
tasks.all_task = Queue()
tasks.task_length = 0
tasks.task_count = 0

#创建crawl_tasks队列
tasks.crawl_task = Queue()

#假性404页面md5列表
conf.autodiscriminator_md5 = set()

bar.log = progressbar.ProgressBar()

def saveResults(domain,msg):
    '''
    @description: 结果保存，以"域名.txt"命名，url去重复
    @param {domain:域名,msg:保存的信息}
    @return: null
    '''
    filename = domain +'.txt'
    conf.output_path = os.path.join(paths.OUTPUT_PATH, filename)
    #判断文件是否存在，若不存在则创建该文件
    if not os.path.exists(conf.output_path):
        with open(conf.output_path,'w+') as temp:
            pass
    with open(conf.output_path,'r+') as result_file:
        old = result_file.read()
        if msg+'\n' in old:
            pass
        else:
            result_file.write(msg+'\n')

def loadConf():
    '''
    @description: 加载扫描配置(以后将使用参数，而非从文件加载)
    @param {type}
    @return:
    '''

    conf.recursive_scan = eval(ConfigFileParser().recursive_scan())
    conf.recursive_scan_max_url_length = eval(ConfigFileParser().recursive_scan_max_url_length())
    conf.recursive_status_code = eval(ConfigFileParser().recursive_status_code())
    conf.recursive_blacklist_exts = eval(ConfigFileParser().recursive_blacklist_exts())
    conf.exclude_subdirs = eval(ConfigFileParser().exclude_subdirs())

    conf.dict_mode = eval(ConfigFileParser().dict_mode())
    conf.dict_mode_load_single_dict = os.path.join(paths.DATA_PATH,eval(ConfigFileParser().dict_mode_load_single_dict()))
    conf.dict_mode_load_mult_dict = os.path.join(paths.DATA_PATH,eval(ConfigFileParser().dict_mode_load_mult_dict()))
    conf.blast_mode = eval(ConfigFileParser().blast_mode())
    conf.blast_mode_min = eval(ConfigFileParser().blast_mode_min())
    conf.blast_mode_max = eval(ConfigFileParser().blast_mode_max())
    conf.blast_mode_az = eval(ConfigFileParser().blast_mode_az())
    conf.blast_mode_num = eval(ConfigFileParser().blast_mode_num())
    conf.blast_mode_custom_charset = eval(ConfigFileParser().blast_mode_custom_charset())
    conf.blast_mode_resume_charset = eval(ConfigFileParser().blast_mode_resume_charset())
    conf.crawl_mode = eval(ConfigFileParser().crawl_mode())
    conf.crawl_mode_dynamic_fuzz_suffix = eval(ConfigFileParser().crawl_mode_dynamic_fuzz_suffix())
    conf.crawl_mode_parse_robots = eval(ConfigFileParser().crawl_mode_parse_robots())
    conf.crawl_mode_parse_html = eval(ConfigFileParser().crawl_mode_parse_html())
    conf.crawl_mode_dynamic_fuzz = eval(ConfigFileParser().crawl_mode_dynamic_fuzz())
    conf.fuzz_mode = eval(ConfigFileParser().fuzz_mode())
    conf.fuzz_mode_load_single_dict = os.path.join(paths.DATA_PATH,eval(ConfigFileParser().fuzz_mode_load_single_dict()))
    conf.fuzz_mode_load_mult_dict = os.path.join(paths.DATA_PATH,eval(ConfigFileParser().fuzz_mode_load_mult_dict()))
    conf.fuzz_mode_label = eval(ConfigFileParser().fuzz_mode_label())

    conf.request_headers = eval(ConfigFileParser().request_headers())
    conf.request_header_ua = eval(ConfigFileParser().request_header_ua())
    conf.request_header_cookie = eval(ConfigFileParser().request_header_cookie())
    conf.request_header_401_auth = eval(ConfigFileParser().request_header_401_auth())
    conf.request_timeout = eval(ConfigFileParser().request_timeout())
    conf.request_delay = eval(ConfigFileParser().request_delay())
    conf.request_limit = eval(ConfigFileParser().request_limit())
    conf.request_max_retries = eval(ConfigFileParser().request_max_retries())
    conf.request_persistent_connect = eval(ConfigFileParser().request_persistent_connect())
    conf.request_method = eval(ConfigFileParser().request_method())
    conf.redirection_302 = eval(ConfigFileParser().redirection_302())
    conf.file_extension = eval(ConfigFileParser().file_extension())

    conf.response_status_code = eval(ConfigFileParser().response_status_code())
    conf.response_header_content_type = eval(ConfigFileParser().response_header_content_type())
    conf.response_size = eval(ConfigFileParser().response_size())
    conf.auto_check_404_page = eval(ConfigFileParser().auto_check_404_page())
    conf.custom_503_page = eval(ConfigFileParser().custom_503_page())
    conf.custom_response_page = eval(ConfigFileParser().custom_response_page())
    conf.skip_size = eval(ConfigFileParser().skip_size())

    conf.proxy_server = eval(ConfigFileParser().proxy_server())

    conf.debug = eval(ConfigFileParser().debug())
    conf.update = eval(ConfigFileParser().update())

def recursiveScan(response_url,all_payloads):
    '''
    @description: 检测出一级目录后，一级目录后遍历添加所有payload，继续检测
    @param {type}
    @return:
    '''
    if not conf.recursive_scan:
        return
    # 当前url后缀在黑名单内，不进行递归
    if response_url.split('.')[-1].lower() in conf.recursive_blacklist_exts:
        return
    #XXX:payloads字典要固定格式
    for payload in all_payloads:
        #判断是否排除。若在排除的目录列表中，则排除。self.excludeSubdirs排除的列表，配置文件中，形如:/test、/test1
        if payload in [directory for directory in conf.exclude_subdirs]:
            return
        #payload拼接，处理/重复或缺失
        if response_url.endswith('/') and payload.startswith('/'):
            # /重复，url和payload都有/，删去payload的/前缀
            payload = payload[1:]
        elif (not response_url.endswith('/')) and (not payload.startswith('/')):
            # /缺失，url和payload都不包含/，在payload前追加/
            payload = '/'+payload
        #拼接payload，限制url长度，入队tasks
        newpayload=response_url+payload
        if(len(newpayload) < int(conf.recursive_scan_max_url_length)):
            tasks.all_task.put(response_url + payload)

def loadSingleDict(path):
    '''
    @description: 添加单个字典文件
    @param {path:字典文件路径}
    @return:
    '''
    try:
        outputscreen.success(f'[+] Load dict:{path}')
        #加载文件时，使用utf-8编码，防止出现编码问题
        with open(path,encoding='utf-8') as single_file:
            return single_file.read().splitlines()
    except Exception as e:
        outputscreen.error(f'[x] plz check file path!\n[x] error:{e}')
        sys.exit()

def loadMultDict(path):
    '''
    @description: 添加多个字典文件
    @param {path:字典文件路径}
    @return:
    '''
    tmp_list = []
    try:
        for file in os.listdir(path):
            #FIXME:这里解决dict和fuzz模式加载多字典问题，但是loadMultDict变得臃肿，后期需要处理
            if conf.dict_mode and conf.fuzz_mode:
                outputscreen.error('[x] Can not use dict and fuzz mode at the same time!')
                sys.exit()
            if conf.dict_mode == 2:
                tmp_list.extend(loadSingleDict(os.path.join(conf.dict_mode_load_mult_dict,file)))
            if conf.fuzz_mode == 2:
                tmp_list.extend(loadSingleDict(os.path.join(conf.fuzz_mode_load_mult_dict,file)))
        return tmp_list
    except  Exception as e:
        outputscreen.error(f'[x] plz check file path!\n[x] error:{e}')
        sys.exit()

def loadSuffix(path):
    '''
    @description: 添加动态爬虫字典后缀规则
    @param {type}
    @return:
    '''
    try:
        with open(path) as f:
            #要去掉#开头的字典
            payloads.suffix = set(f.read().split('\n')) - {'', '#'}
    except  Exception as e:
        outputscreen.error(f'[x] plz check file path!\n[x] error:{e}')
        sys.exit()

def generateCrawlDict(base_url):
    '''
    @description: 生成动态爬虫字典
    @param {base_url:}
    @return:
    '''
    def _splitFilename(filename):

        full_filename = filename.rstrip('.')
        extension = full_filename.split('.')[-1]
        name = '.'.join(full_filename.split('.')[:-1])

        return name, extension

    url = base_url.split('?')[0].rstrip('/')
    if not urllib.parse.urlparse(url).path:
        return list()

    path = '/'.join(url.split('/')[:-1])
    filename = url.split('/')[-1]

    # Check if target CMS uses route instead of static file
    isfile = True if '.' in filename else False

    if isfile:
        name, extension = _splitFilename(filename)

    final_urls = list()
    for each in payloads.suffix:
        new_filename = path + '/' + each.replace('{FULL}', filename)
        if isfile:
            new_filename = new_filename.replace('{NAME}', name).replace('{EXT}', extension)
        else:
            if '{NAME}' in each or '{EXT}' in each:
                continue
        final_urls.append(urllib.parse.urlparse(new_filename.replace('..', '.')).path)

    return final_urls

def generateBlastDict():
    '''
    @description: 生成纯暴力字典，支持断点续生成
    @param {type}
    @return:
    '''
    if conf.blast_mode_min > conf.blast_mode_max:
        outputscreen.error("[x] The minimum length should be less than or equal to the maximum length")
        sys.exit(1)
    the_min = conf.blast_mode_min
    if conf.blast_mode_resume_charset != '':
        the_min = len(conf.blast_mode_resume_charset)
        if conf.blast_mode_min > the_min or conf.blast_mode_max < the_min:
            outputscreen.error('[+] Invalid resume length: %d\n\n' % the_min)
            the_min = conf.blast_mode_min
            conf.blast_mode_resume_charset = ''
    for length in range(the_min, conf.blast_mode_max + 1):
        generateLengthDict(length)
        conf.blast_mode_resume_charset = ''
    return payloads.blast_mode_custom_charset_dict

def generateLengthDict(length):
    '''
    @description: 生成length长度的字典
    @param {type}
    @return:
    '''
    lst = [0] * length
    if len(conf.blast_mode_resume_charset) == length and conf.blast_mode_resume_charset != '':
        #enumerate()用于将一个可遍历的数据对象(如列表、元组或字符串)组合为一个索引序列
        for i, letter in enumerate(conf.blast_mode_resume_charset):
            if conf.blast_mode_custom_charset.find(letter) == -1:
                outputscreen.error('[+] Invalid resume string: "%s"\n\n' % conf.blast_mode_resume_charset)
                lst = [0] * length
                break
            lst[i] = conf.blast_mode_custom_charset.find(letter)
    lines_max = 1
    for l in lst:
        lines_max *= (len(conf.blast_mode_custom_charset) - l)
    i = length - 1
    print_it = True
    while i >= 0:
        if print_it:
            temp = ''
            for j in lst:
                temp += conf.blast_mode_custom_charset[j]
            payloads.blast_mode_custom_charset_dict.append(temp)
            print_it = False
        lst[i] += 1
        if lst[i] >= len(conf.blast_mode_custom_charset):
            lst[i] = 0
            i -= 1
        else:
            i = length - 1
            print_it = True

def generateSingleFuzzDict(path):
    '''
    @description: 单字典。生成fuzz字典
    @param {type}
    @return:
    '''
    fuzz_path = urllib.parse.urlparse(conf.url).path
    #替换label进行fuzz字典生成
    if conf.fuzz_mode_label in fuzz_path:
        for i in loadSingleDict(path):
            payloads.fuzz_mode_dict.append(fuzz_path.replace(conf.fuzz_mode_label,i))
        return payloads.fuzz_mode_dict
    else:
        outputscreen.error("[x] Please set the fuzz label")
        sys.exit(1)
def generateMultFuzzDict(path):
    '''
    @description: 多字典。生成fuzz字典
    @param {type}
    @return:
    '''
    fuzz_path = urllib.parse.urlparse(conf.url).path
    #替换label进行fuzz字典生成
    if conf.fuzz_mode_label in fuzz_path:
        for i in loadMultDict(path):
            payloads.fuzz_mode_dict.append(fuzz_path.replace(conf.fuzz_mode_label,i))
        return payloads.fuzz_mode_dict
    else:
        outputscreen.error("[x] Please set the fuzz label")
        sys.exit(1)

def scanModeHandler():
    '''
    @description: 扫描模式处理，加载payloads
    @param {type}
    @return:
    '''
    if conf.recursive_scan:
        msg = '[*] Use recursive scan: Yes'
        outputscreen.warning('\r'+msg+' '*(th.console_width-len(msg)+1))
    else:
        msg = '[*] Use recursive scan: No'
        outputscreen.warning('\r'+msg+' '*(th.console_width-len(msg)+1))
    payloadlists=[]
    # fuzz模式处理，只能单独加载
    if conf.fuzz_mode:
        outputscreen.warning('[*] Use fuzz mode')
        if conf.fuzz_mode == 1:
            return generateSingleFuzzDict(conf.fuzz_mode_load_single_dict)
        if conf.fuzz_mode == 2:
            return generateMultFuzzDict(conf.fuzz_mode_load_mult_dict)
    # 其他模式处理，可同时加载
    else:
        if conf.dict_mode:
            outputscreen.warning('[*] Use dict mode')
            if conf.dict_mode == 1:
                payloadlists.extend(loadSingleDict(conf.dict_mode_load_single_dict))
            elif conf.dict_mode == 2:
                payloadlists.extend(loadMultDict(conf.dict_mode_load_mult_dict))
            else:
                outputscreen.error("[-] You must select a dict")
                sys.exit()
        if conf.blast_mode:
            outputscreen.warning('[*] Use blast mode')
            outputscreen.warning(f'[*] Use char set: {conf.blast_mode_custom_charset}')
            outputscreen.warning(f'[*] Use paylaod min length: {conf.blast_mode_min}')
            outputscreen.warning(f'[*] Use paylaod max length: {conf.blast_mode_max}')
            payloadlists.extend(generateBlastDict())
        #TODO:递归爬取url
        if conf.crawl_mode:
            outputscreen.warning('[*] Use crawl mode')
            #自定义header
            headers = {}
            if conf.request_headers:
                try:
                    for header in conf.request_headers.split(','):
                        k, v = header.split('=')
                        #print(k,v)
                        headers[k] = v
                except Exception as e:
                    outputscreen.error(f"[x] Check personalized headers format: header=value,header=value.\n[x] error:{e}")
                    # sys.exit()
            #自定义ua
            if conf.request_header_ua:
                headers['User-Agent'] = conf.request_header_ua
            #自定义cookie
            if conf.request_header_cookie:
                headers['Cookie'] = conf.request_header_cookie
            try:
                response = requests.get(conf.url, headers=headers, timeout=conf.request_timeout, verify=False, allow_redirects=conf.redirection_302, proxies=conf.proxy_server)
                #获取页面url
                if (response.status_code in conf.response_status_code) and response.text:
                    html = etree.HTML(response.text)
                    #加载自定义xpath用于解析html
                    urls = html.xpath(conf.crawl_mode_parse_html)
                    for url in urls:
                        #去除相似url
                        if urlSimilarCheck(url):
                            #判断:1.是否同域名 2.netloc是否为空(值空时为同域)。若满足1或2，则添加到temp payload
                            if (urllib.parse.urlparse(url).netloc == urllib.parse.urlparse(conf.url).netloc) or urllib.parse.urlparse(url).netloc == '':
                                payloads.crawl_mode_dynamic_fuzz_temp_dict.add(url)
                payloads.crawl_mode_dynamic_fuzz_temp_dict = payloads.crawl_mode_dynamic_fuzz_temp_dict - {'#', ''}
                if conf.crawl_mode_dynamic_fuzz:
                    #加载动态fuzz后缀，TODO:独立动态生成字典模块
                    loadSuffix(os.path.join(paths.DATA_PATH,conf.crawl_mode_dynamic_fuzz_suffix))
                    #生成新爬虫动态字典
                    for i in payloads.crawl_mode_dynamic_fuzz_temp_dict:
                        payloads.crawl_mode_dynamic_fuzz_dict.extend(generateCrawlDict(i))
                    for i in payloads.crawl_mode_dynamic_fuzz_temp_dict:
                        payloads.crawl_mode_dynamic_fuzz_dict.append(urllib.parse.urlparse(i).path)
                    payloadlists.extend(set(payloads.crawl_mode_dynamic_fuzz_dict))
                else:
                    for i in payloads.crawl_mode_dynamic_fuzz_temp_dict:
                        payloads.crawl_mode_dynamic_fuzz_dict.append(urllib.parse.urlparse(i).path)
                    payloadlists.extend(set(payloads.crawl_mode_dynamic_fuzz_dict))
            except (requests.exceptions.ConnectionError, requests.exceptions.Timeout, requests.exceptions.ReadTimeout) as e:
                outputscreen.error(f"[x] Crawler network connection error!plz check whether the target is accessible, error info:{e}")
                # sys.exit()

    if payloadlists:
        return payloadlists
    else:
        outputscreen.error("[-] You have to select at least one mode , plz check mode config")
        sys.exit()

def responseHandler(response):
    '''
    @description: 处理响应结果
    @param {type}
    @return:
    '''
    #结果处理阶段
    try:
        size = intToSize(int(response.headers['content-length']))
    except (KeyError, ValueError):
        size = intToSize(len(response.content))
    #跳过大小为skip_size的页面
    if size == conf.skip_size:
        return

    #自动识别404-判断是否与获取404页面特征匹配
    if conf.auto_check_404_page:
        if hashlib.md5(response.content).hexdigest() in conf.autodiscriminator_md5:
            return

    #自定义状态码显示
    if response.status_code in conf.response_status_code:
        msg = f'[{str(response.status_code)}]'
        if conf.response_header_content_type:
            msg += '[{}]'.format(response.headers.get('content-type'))
        if conf.response_size:
            msg += f'[{str(size)}] '
        msg += response.url
        if response.status_code in [200]:
            outputscreen.success('\r'+msg+' '*(th.console_width-len(msg)+1))
        elif response.status_code in [301,302]:
            outputscreen.warning('\r'+msg+' '*(th.console_width-len(msg)+1))
        elif response.status_code in [403,404]:
            outputscreen.error('\r'+msg+' '*(th.console_width-len(msg)+1))
        else:
            outputscreen.info('\r'+msg+' '*(th.console_width-len(msg)+1))
        #已去重复，结果保存。NOTE:此处使用response.url进行文件名构造，解决使用-iL参数时，不能按照域名来命名文件名的问题
        #使用replace()，替换`:`，修复window下不能创建有`:`的文件问题
        saveResults(urllib.parse.urlparse(response.url).netloc.replace(':','_'),msg)
    #关于递归扫描。响应在自定义状态码中时，添加判断是否进行递归扫描
    if response.status_code in conf.recursive_status_code:
        if conf.recursive_scan:
            recursiveScan(response.url,payloads.all_payloads)

    #自定义正则匹配响应
    if conf.custom_response_page:
        pattern = re.compile(conf.custom_response_page)
        if pattern.search(response.text):
            outputscreen.info(f'[!] Custom response information matched\n[!] use regular expression:{conf.custom_response_page}\n[!] matched page:{response.text}')

def worker():
    '''
    @description: 封包发包穷举器
    @param {type}
    @return:
    '''
    payloads.current_payload = tasks.all_task.get()
    #1自定义封包阶段
    #自定义header
    headers = {}
    if conf.request_headers:
        try:
            for header in conf.request_headers.split(','):
                k, v = header.split('=')
                #print(k,v)
                headers[k] = v
        except Exception as e:
            outputscreen.error(f"[x] Check personalized headers format: header=value,header=value.\n[x] error:{e}")
            sys.exit()
    #自定义ua
    if conf.request_header_ua:
        headers['User-Agent'] = conf.request_header_ua
    #自定义cookie
    if conf.request_header_cookie:
        headers['Cookie'] = conf.request_header_cookie

    try:
        #2进入发送请求流程
        #延迟请求
        if conf.request_delay:
            random_sleep_second = random.randint(0,abs(conf.request_delay))
            time.sleep(random_sleep_second)

        response = requests.request(conf.request_method, payloads.current_payload, headers=headers, timeout=conf.request_timeout, verify=False, allow_redirects=conf.redirection_302, proxies=conf.proxy_server)
        #3进入结果处理流程
        responseHandler(response)
    except requests.exceptions.Timeout:
        #outputscreen.error('[x] timeout! url:{}'.format(payloads.current_payload))
        pass
    except Exception:
        # outputscreen.error('[x] error:{}'.format(e))
        pass
    finally:
        #更新进度条
        tasks.task_count += 1
        bar.log.update(tasks.task_count)

def boss():
    '''
    @description: worker控制器
    @param {type}
    @return:
    '''
    while not tasks.all_task.empty():
        worker()

def bruter(url):
    '''
    @description: 扫描插件入口函数
    @param {url:目标}
    @return:
    '''

    #url初始化
    conf.parsed_url = urllib.parse.urlparse(url)
    #填补协议
    if conf.parsed_url.scheme != 'http' and conf.parsed_url.scheme != 'https':
        url = 'http://' + url
        conf.parsed_url = urllib.parse.urlparse(url)
    #全局target的url，给crawl、fuzz模块使用。XXX:要放在填补url之前，否则fuzz模式会出现这样的问题：https://target.com/phpinfo.{dir}/
    conf.url = url
    #填补url后的/
    if not url.endswith('/'):
        url = url + '/'

    #打印当前target
    msg = f'[+] Current target: {url}'
    outputscreen.success('\r'+msg+' '*(th.console_width-len(msg)+1))
    #自动识别404-预先获取404页面特征
    if conf.auto_check_404_page:
        outputscreen.warning("[*] Launching auto check 404")
        # Autodiscriminator (probably deprecated by future diagnostic subsystem)
        i = Inspector(url)
        (result, notfound_type) = i.check_this()
        if notfound_type == Inspector.TEST404_MD5 or notfound_type == Inspector.TEST404_OK:
            conf.autodiscriminator_md5.add(result)

    #加载payloads
    payloads.all_payloads = scanModeHandler()
    #FIXME:设置后缀名。当前以拼接方式实现，遍历一遍payload。
    try:
        if conf.file_extension:
            outputscreen.warning(f'[+] Use file extentsion: {conf.file_extension}')
            for i in range(len(payloads.all_payloads)):
                payloads.all_payloads[i] += conf.file_extension
    except:
        outputscreen.error('[+] plz check extension!')
        sys.exit()
    #debug模式，打印所有payload，并退出
    if conf.debug:
        outputscreen.blue(f'[+] all payloads:{payloads.all_payloads}')
        sys.exit()
    #payload入队task队列
    for payload in payloads.all_payloads:
        #FIXME:添加fuzz模式时，引入的url_payload构造判断
        if conf.fuzz_mode:
            url_payload = conf.parsed_url.scheme + '://' + conf.parsed_url.netloc + payload
        else:
            url_payload = url + payload
        #print(url_payload)
        #payload入队，等待处理
        tasks.all_task.put(url_payload)
    #设置进度条长度，若是递归模式或爬虫模式，则不设置任务队列长度，即无法显示进度，仅显示耗时
    if not conf.recursive_scan:
        #NOTE:这里取所有payloads的长度*target数量计算任务总数，修复issue#2
        tasks.task_length = len(payloads.all_payloads)*conf.target_nums
        bar.log.start(tasks.task_length)
    #FIXME:循环任务数不能一次性取完所有的task，暂时采用每次执行30个任务。这样写还能解决hub.LoopExit的bug
    while not tasks.all_task.empty():
        all_task = [gevent.spawn(boss) for i in range(conf.request_limit)]
        gevent.joinall(all_task)



#!/usr/bin/env python

# Release process:
#
#  - set version in IPy.py
#  - set version in setup.py
#  - run unit test: make
#  - run unit test: make PYTHON=python3
#  - set release date in ChangeLog
#  - git commit -a
#  - git tag -a IPy-x.y -m "tag IPy x.y"
#  - git push
#  - git push --tags
#  - python setup.py register sdist upload
#  - update the website
#
# After the release:
#  - set version to n+1 (IPy.py and setup.py)
#  - add a new empty section in the changelog for version n+1
#  - git commit -a
#  - git push

from distutils.core import setup

VERSION = '0.83'

options = {}

with open('README') as fp:
    README = fp.read().strip() + "\n\n"

ChangeLog = (
    "What's new\n"
    "==========\n"
    "\n")
with open('ChangeLog') as fp:
    ChangeLog += fp.read().strip()

LONG_DESCRIPTION = README + ChangeLog
CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Environment :: Plugins',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Communications',
    'Topic :: Internet',
    'Topic :: System :: Networking',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Natural Language :: English',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
]
URL = "https://github.com/autocracy/python-ipy"

setup(
    name="IPy",
    version=VERSION,
    description="Class and tools for handling of IPv4 and IPv6 addresses and networks",
    long_description=LONG_DESCRIPTION,
    author="Maximillian Dornseif",
    maintainer="Jeff Ferland",
    maintainer_email="jeff AT storyinmemo.com",
    license="BSD License",
    keywords="ipv4 ipv6 netmask",
    url=URL,
    download_url=URL,
    classifiers= CLASSIFIERS,
    py_modules=["IPy"],
    **options
)




#!/usr/bin/env python3
"""
自动升级 astrbot/ 文件夹下所有 Python 文件的脚本。
使用 pyupgrade 将代码升级到 Python 3.10+ 兼容版本。
"""

import os
import subprocess
from pathlib import Path

def main():
    # 获取当前脚本目录
    script_dir = Path(__file__).resolve().parent
    
    # 确定 astrbot/ 文件夹的位置
    src_dir = script_dir.parent / "src"
    

    print(f"开始处理 {src_dir} 目录下的 Python 文件...")
    
    # 计数器，记录处理的文件数量
    processed_count = 0
    
    # 遍历 src_dir/ 目录及其子目录下的所有 .py 文件
    for root, _, files in os.walk(src_dir):
        for file in files:
            if file.endswith('.py'):
                file_path = Path(root) / file
                try:
                    # 对每个 .py 文件执行 pyupgrade 命令
                    print(f"正在处理: {file_path}")
                    result = subprocess.run(
                        ["pyupgrade", "--py313-plus", str(file_path)],
                        capture_output=True,
                        text=True,
                        check=False
                    )
                    
                    if result.returncode == 0:
                        processed_count += 1
                        print(f"✓ 成功升级: {file_path}")
                    else:
                        print(f"✗ 处理失败: {file_path}")
                        print(f"错误信息: {result.stderr.strip()}")
                except Exception as e:
                    print(f"处理 {file_path} 时发生错误: {e}")
    
    print(f"\n处理完成! 共处理了 {processed_count} 个 Python 文件。")
    print("如果您没有看到任何文件被处理，请确认您已安装 pyupgrade：")
    print("uv tool install pyupgrade")

if __name__ == "__main__":
    main()


#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: xxlin
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-05-01 20:19:40
'''

import gevent
import time
import traceback
from ..core.data import conf,th
from ..core.common import outputscreen
from ..utils.console import getTerminalSize
from ..controller.bruter import bruter

def initEngine():
    # init control parameter
    th.result = []
    th.thread_num = conf.thread_num
    th.target = conf.target
    #是否继续扫描标志位
    th.is_continue = True
    #控制台宽度
    th.console_width = getTerminalSize()[0] - 2
    #记录开始时间
    th.start_time = time.time()
    msg = '[+] Set the number of thread: %d' % th.thread_num
    outputscreen.success(msg)

def scan():
    while True:
        #协程模式
        if th.target.qsize() > 0 and th.is_continue:
            target = str(th.target.get(timeout=1.0))
        else:
            break
        try:
            #对每个target进行检测
            bruter(target)
        except Exception:
            #抛出异常时，添加errmsg键值
            th.errmsg = traceback.format_exc()
            th.is_continue = False

def run():
    initEngine()
    # Coroutine mode
    outputscreen.success('[+] Coroutine mode')
    gevent.joinall([gevent.spawn(scan) for i in range(0, th.thread_num)])
    if 'errmsg' in th:
        outputscreen.error(th.errmsg)



#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: ttttmr
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-07-22 21:46:31
'''

import sys
import argparse

def cmdLineParser() -> argparse.Namespace:
    """
    This function parses the command line parameters and arguments
    """
    parser = argparse.ArgumentParser(usage="dirmap -i https://target.com -lcf")

    # engine
    engine = parser.add_argument_group("Engine", "Engine config")
    engine.add_argument("-t","--thread",  dest="thread_num", type=int, default=30,
                        help="num of threads, default 30")

    # target
    target = parser.add_argument_group("Target","Target config")
    target.add_argument("-i", metavar="TARGET", dest="target_input", type=str, default="",
                        help="scan a target or network (e.g. [http://]target.com , 192.168.1.1[/24] , 192.168.1.1-192.168.1.100)")
    target.add_argument("-iF", metavar="FILE", dest="target_file", type=str, default="",
                        help="load targets from targetFile (e.g. urls.txt)")

    # bruter
    bruter = parser.add_argument_group("Bruter", "Bruter config")
    bruter.add_argument("-lcf", "--loadConfigFile",  dest="load_config_file", default=True, action="store_true",
                        help="Load the configuration through the configuration file")
    bruter.add_argument("--debug", dest="debug",default=False, action="store_true",
                        help="Print payloads and exit")

    if len(sys.argv) == 1:
        sys.argv.append("-h")
    args = parser.parse_args()
    return args



#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: ttttmr
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-05-29 16:49:22
'''

import os.path
import sys
import urllib
import ipaddress
import re
from typing import Any

from ..core.data import paths, payloads
from ..core.setting import BANNER
from ...thirdlib.colorama import Fore, Style, init

init(autoreset=True)
class Outputscreen:
    """
    显示颜色类
    """
    def info(self, s: Any) -> None:
        print(Style.BRIGHT+Fore.WHITE + str(s) + Fore.RESET+Style.RESET_ALL)

    def success(self, s: Any) -> None:
        print(Style.BRIGHT+Fore.GREEN + str(s) + Fore.RESET+Style.RESET_ALL)

    def warning(self, s: Any) -> None:
        print(Style.BRIGHT+Fore.CYAN + str(s) + Fore.RESET+Style.RESET_ALL)

    def error(self, s: Any) -> None:
        print(Style.BRIGHT+Fore.RED + str(s) + Fore.RESET+Style.RESET_ALL)

    # for banner
    def blue(self, s: Any) -> None:
        print(Style.BRIGHT+Fore.BLUE + str(s) + Fore.RESET+Style.RESET_ALL)

#创建outputscreen对象，用于输出各种颜色的信息
outputscreen=Outputscreen()

def setPaths() -> None:
    """
    设置全局绝对路径
    """
    # 根目录
    root_path = paths.ROOT_PATH
    # datapath
    paths.DATA_PATH = os.path.join(root_path, "data")
    paths.OUTPUT_PATH = os.path.join(root_path, "output")
    paths.CONFIG_PATH = os.path.join(root_path, "dirmap.conf")
    if not os.path.exists(paths.OUTPUT_PATH):
        os.mkdir(paths.OUTPUT_PATH)
    if not os.path.exists(paths.DATA_PATH):
        os.mkdir(paths.DATA_PATH)

    # paths.WEAK_PASS = os.path.join(paths.DATA_PATH, "pass100.txt")
    # paths.LARGE_WEAK_PASS = os.path.join(paths.DATA_PATH, "pass1000.txt")
    # paths.UA_LIST_PATH = os.path.join(paths.DATA_PATH, "user-agents.txt")

    if os.path.isfile(paths.CONFIG_PATH):
        pass
    else:
        msg = 'Config files missing, it may cause an issue.\n'
        outputscreen.error(msg)
        sys.exit(0)

    #print(root_path,paths.DATA_PATH,paths.SCRIPT_PATH,paths.OUTPUT_PATH,paths.CONFIG_PATH)
    #print(paths.WEAK_PASS,paths.LARGE_WEAK_PASS,paths.UA_LIST_PATH)

def banner() -> None:
    '''
    @description: 打印banner
    @param {type}
    @return:
    '''
    outputscreen.blue(BANNER)

# 将'192.168.1.1-192.168.1.100'分解成ip地址列表
def genIP(ip_range: str) -> list[str]:
    '''
    print (genIP('192.18.1.1-192.168.1.3'))
    ['192.168.1.1', '192.168.1.2', '192.168.1.3']
    '''
    # from https://segmentfault.com/a/1190000010324211
    def num2ip (num: int) -> str:
        return f'{(num >> 24) & 0xff}.{(num >> 16) & 0xff}.{(num >> 8) & 0xff}.{(num & 0xff)}'

    def ip2num(ip: str) -> int:
        ips = [int(x) for x in ip.split('.')]
        return ips[0]<< 24 | ips[1]<< 16 | ips[2] << 8 | ips[3]

    start ,end = (ip2num(x) for x in ip_range.split('-'))
    return [num2ip(num) for num in range(start,end+1) if num & 0xff]

# 识别目标，转换成列表形式
def parseTarget(target: str) -> list[str]:
    lists=[]
    ipv4withmask_re=re.compile(r"^(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])/(3[0-2]|[1-2]?[0-9])$")
    ipv4range_re=re.compile(r"^(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])-(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])$")
    try:
        # 尝试解析url
        parsed_url=urllib.parse.urlparse(target)
        # 判断不带http
        if parsed_url.scheme != 'http':
            # 判断IP/Mask格式
            if ipv4withmask_re.search(parsed_url.path):
                # 是子网还是网址 e.g. 192.168.1.1/24 or http://192.168.1.1/24
                infomsg = "[*] %s is IP/Mask[Y] or URL(http://)[n]? [Y/n]" %target
                outputscreen.info(infomsg)
                flag =input()
                if flag in ('N', 'n', 'no', 'No', 'NO'):
                    # 按链接处理 e.g. http://192.168.1.1/24
                    lists.append(target)
                else:
                    # 按子网处理 e.g. 192.168.1.1/24
                    lists=list(map(str, ipaddress.ip_interface(target).network))
            # 判断网络范围格式 e.g. 192.168.1.1-192.168.1.100
            elif ipv4range_re.search(target):
                lists=genIP(target)
            # 按照链接处理
            else:
                lists.append(target)
        # 为http://格式
        else:
            lists.append(target)
    except:
        # 识别失败
        pass
    return lists

def intToSize(bytes: int) -> str:
    '''
    @description: bits大小转换，对人类友好
    @param {type}
    @return:
    '''
    b = 1024 * 1024 * 1024 * 1024
    a = ['t','g','m','k','']
    for i in a:
        if bytes >= b:
            return f'{float(bytes) / float(b):.2f}{i}b'
        b /= 1024
    return '0b'

def urlSimilarCheck(url: str) -> bool:
    '''
    @description: url相似度分析，当url路径和参数键值类似时，则判为重复，参考某人爬虫
    @param {type}
    @return: 非重复返回True
    '''
    url_struct = urllib.parse.urlparse(url)
    query_key = '|'.join(sorted([i.split('=')[0] for i in url_struct.query.split('&')]))
    url_hash = hash(url_struct.path + query_key)
    if url_hash not in payloads.similar_urls_set:
        payloads.similar_urls_set.add(url_hash)
        return True
    return False



#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: xxlin
@Date: 2019-04-10 13:27:59
@LastEditTime: 2023-07-25 15:56:04
'''

from pathlib import Path
import sys

from gevent import monkey
monkey.patch_all()
from .lib.controller.engine import run
from .lib.core.common import banner, outputscreen, setPaths
from .lib.core.data import cmdLineOptions, paths
from .lib.core.option import initOptions
from .lib.parse.cmdline import cmdLineParser




def main() -> None:
    """
    main fuction of dirmap 
    """
    # Make sure the version of python you are using is high enough
    if sys.version_info < (3, 8):
        outputscreen.error("Sorry, dirmap requires Python 3.8 or higher\n")
        sys.exit(1)
    # anyway output thr banner information
    banner() 

    # set paths of project 
    paths.ROOT_PATH = str(Path(__file__).parent)
    print(f" 输出：{Path(paths.ROOT_PATH) / 'output'}   \n 数据：{Path(paths.ROOT_PATH) / 'data' }  \n 配置：{Path(paths.ROOT_PATH) / 'dirmap.conf'}"  )
    setPaths()
    
    # received command >> cmdLineOptions
    cmdLineOptions.update(cmdLineParser().__dict__)
    
    # loader script,target,working way(threads? gevent?),output_file from cmdLineOptions
    # and send it to conf
    initOptions(cmdLineOptions) 

    # run!
    try:
        run()
    except KeyboardInterrupt:
        outputscreen.success("[+] exit")
        sys.exit(1)

if __name__ == "__main__":
    main()



# Copyright Jonathan Hartley 2013. BSD 3-Clause license, see LICENSE file.
from .initialise import init, deinit, reinit, colorama_text
from .ansi import Fore, Back, Style, Cursor
from .ansitowin32 import AnsiToWin32

__version__ = '0.4.1'

__all__ = ['init', 'deinit', 'reinit', 'colorama_text', 'Fore', 'Back', 'Style', 'Cursor', 'AnsiToWin32']


#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: xxlin
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-04-14 10:36:27
'''

class COLOR:
    black = 30  #  黑色
    red = 31  #  红色
    green = 32  #  绿色
    yellow = 33  #  黄色
    blue = 34  #  蓝色
    purple = 35  #  紫红色
    cyan = 36  #  青蓝色
    white = 37  #  白色
    
class BRUTER_RESULT_STATUS:
    FAIL = 0
    SUCCESS = 1
    RETRAY = 2



#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: ttttmr
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-05-29 16:52:42
'''

import os
import queue
import sys

from ..controller.bruter import loadConf
from ..core.common import parseTarget, outputscreen
from ..core.data import conf

def initOptions(args):
    EngineRegister(args)
    BruterRegister(args)
    TargetRegister(args)


def EngineRegister(args):
    """
    加载并发引擎模块
    """
    conf.engine_mode = 'coroutine'

    #设置线程数
    if args.thread_num > 200 or args.thread_num < 1:
        msg = '[*] Invalid input in [-t](range: 1 to 200), has changed to default(30)'
        outputscreen.warning(msg)
        conf.thread_num = 30
        return
    conf.thread_num = args.thread_num

def BruterRegister(args):
    """
    配置bruter模块
    """

    if args.load_config_file:
        #加载配置文件
        loadConf()
    else:
        outputscreen.error("[+] Function development, coming soon!please use -lcf parameter")
        if args.debug:
            conf.debug = args.debug
        else:
            conf.debug = args.debug
        sys.exit()

def TargetRegister(args):
    """
    加载目标模块
    """
    msg = '[*] Initialize targets...'
    outputscreen.warning(msg)

    #初始化目标队列
    conf.target = queue.Queue()

    # 用户输入入队
    if args.target_input:
        # 尝试解析目标地址
        try:
            lists=parseTarget(args.target_input)
        except:
            helpmsg = "Invalid input in [-i], Example: -i [http://]target.com or 192.168.1.1[/24] or 192.168.1.1-192.168.1.100"
            outputscreen.error(helpmsg)
            sys.exit()
        # 判断处理量
        if (len(lists))>100000:
            warnmsg = "[*] Loading %d targets, Maybe it's too much, continue? [y/N]" % (len(lists))
            outputscreen.warning(warnmsg)
            flag =input()
            if flag in ('Y', 'y', 'yes', 'YES','Yes'):
                pass
            else:
                msg = '[-] User quit!'
                outputscreen.warning(msg)
                sys.exit()
        msg = '[+] Load targets from: %s' % args.target_input
        outputscreen.success(msg)
        # save to conf
        for target in lists:
            conf.target.put(target)
        conf.target_nums = conf.target.qsize()

    # 文件读入入队
    elif args.target_file:
        if not os.path.isfile(args.target_file):
            msg = '[-] TargetFile not found: %s' % args.target_file
            outputscreen.error(msg)
            sys.exit()
        msg = '[+] Load targets from: %s' % args.target_file
        outputscreen.success(msg)
        with open(args.target_file, encoding='utf-8') as f:
            targets = f.readlines()
            for target in targets:
                target=target.strip('\n')
                parsed_target=parseTarget(target)
                for i in parsed_target:
                    conf.target.put(i)
        conf.target_nums = conf.target.qsize()

    #验证目标数量
    if conf.target.qsize() == 0:
        errormsg = msg = '[!] No targets found.Please load targets with [-i|-iF]'
        outputscreen.error(errormsg)
        sys.exit()


"""Fuzing for IPy.py"""

# TODO: unify assert / FilIf usage

import sys
import functools
import itertools
sys.path.append('.')
sys.path.append('..')

import IPy
import unittest
import random

xrange = range

# on Python-2.7 and higher, we use load_tests to multiply out the test cases so that unittest
# represents each as an individual test case.
def iterate_27(n):
    def wrap(func):
        func.iterations = n
        return func
    return wrap

def load_tests(loader, tests, pattern):
    def expand(tests):
        if isinstance(tests, unittest.TestCase):
            method_name = tests._testMethodName
            meth = getattr(tests, method_name)
            if hasattr(meth, 'iterations'):
                tests = unittest.TestSuite(type(tests)(method_name) for i in xrange(meth.iterations))
        else:
            tests = unittest.TestSuite(expand(t) for t in tests)
        return tests
    return expand(tests)

# On older Pythons, we run the requisite iterations directly, in a single test case.
def iterate_old(n):
    def wrap(func):
        @functools.wraps(func)
        def replacement(*args):
            for i in xrange(n):
                func(*args)
        return replacement
    return wrap

if sys.version_info >= (2,7):
    iterate = iterate_27
else:
    iterate = iterate_old

# utilities

def random_ipv4_prefix():
    prefixlen = random.randrange(32)
    int_ip = random.randrange(IPy.MAX_IPV4_ADDRESS)
    int_ip &= 0xffffffff << (32-prefixlen)
    return IPy.IP('.'.join(map(str, (int_ip >> 24,
                                    (int_ip >> 16) & 0xff,
                                    (int_ip >> 8) & 0xff,
                                    int_ip & 0xff)))
                           + '/%d' % prefixlen)

# tests

class ParseAndBack(unittest.TestCase):

    @iterate(500)
    def testRandomValuesv4(self):
        question = random.randrange(0xffffffff)
        self.assertEqual(IPy.parseAddress(IPy.intToIp(question, 4)), (question, 4), hex(question))

    @iterate(500)
    def testRandomValuesv6(self):
        question = random.randrange(0xffffffffffffffffffffffffffffffff)
        self.assertEqual(IPy.parseAddress(IPy.intToIp(question, 6)), (question, 6), hex(question))

class TestIPSet(unittest.TestCase):

    @iterate(1000)
    def testRandomContains(self):
        prefixes = [random_ipv4_prefix() for i in xrange(random.randrange(50))]
        question = random_ipv4_prefix()
        answer = any(question in pfx for pfx in prefixes)
        ipset = IPy.IPSet(prefixes)
        self.assertEqual(question in ipset, answer,
                f"{question} in {ipset} != {answer} (made from {prefixes})")
        

    @iterate(1000)
    def testRandomDisjoint(self):
        prefixes1 = [random_ipv4_prefix() for i in xrange(random.randrange(50))]
        prefixes2 = [random_ipv4_prefix() for i in xrange(random.randrange(50))]
        # test disjointnes the stupid way
        disjoint = True
        for p1, p2 in itertools.product(prefixes1, prefixes2):
            if p1 in p2 or p2 in p1:
                disjoint = False
                break
        ipset1 = IPy.IPSet(prefixes1)
        ipset2 = IPy.IPSet(prefixes2)
        self.assertEqual(ipset1.isdisjoint(ipset2), disjoint,
                f"{ipset1}.isdisjoint({ipset2}) != {disjoint}")
        self.assertEqual(ipset2.isdisjoint(ipset1), disjoint,
                f"{ipset2}.isdisjoint({ipset1}) != {disjoint}")

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: ttttmr
@Date: 2019-04-11 09:49:16
@LastEditTime: 2019-05-29 16:49:43
'''

from configparser import ConfigParser
from ..core.data import paths
from ..core.common import outputscreen


class ConfigFileParser:
    @staticmethod
    def _get_option(section: str, option: str) -> str:
        try:
            cf = ConfigParser()
            cf.read(paths.CONFIG_PATH)
            return cf.get(section=section, option=option)
        except:
            outputscreen.warning('Missing essential options, please check your config-file.')
            return ''


    def recursive_scan(self) -> str:
        return self._get_option('RecursiveScan','conf.recursive_scan')
    def recursive_status_code(self) -> str:
        return self._get_option('RecursiveScan','conf.recursive_status_code')
    def recursive_scan_max_url_length(self) -> str:
        return self._get_option('RecursiveScan','conf.recursive_scan_max_url_length')
    def recursive_blacklist_exts(self) -> str:
        return self._get_option('RecursiveScan','conf.recursive_blacklist_exts')
    def exclude_subdirs(self) -> str:
        return self._get_option('RecursiveScan','conf.exclude_subdirs')

    def dict_mode(self) -> str:
        return self._get_option('ScanModeHandler','conf.dict_mode')
    def dict_mode_load_single_dict(self) -> str:
        return self._get_option('ScanModeHandler','conf.dict_mode_load_single_dict')
    def dict_mode_load_mult_dict(self) -> str:
        return self._get_option('ScanModeHandler','conf.dict_mode_load_mult_dict')
    def blast_mode(self) -> str:
        return self._get_option('ScanModeHandler','conf.blast_mode')
    def blast_mode_min(self) -> str:
        return self._get_option('ScanModeHandler','conf.blast_mode_min')
    def blast_mode_max(self) -> str:
        return self._get_option('ScanModeHandler','conf.blast_mode_max')
    def blast_mode_az(self) -> str:
        return self._get_option('ScanModeHandler','conf.blast_mode_az')
    def blast_mode_num(self) -> str:
        return self._get_option('ScanModeHandler','conf.blast_mode_num')
    def blast_mode_custom_charset(self) -> str:
        return self._get_option('ScanModeHandler','conf.blast_mode_custom_charset')
    def blast_mode_resume_charset(self) -> str:
        return self._get_option('ScanModeHandler','conf.blast_mode_resume_charset')
    def crawl_mode(self) -> str:
        return self._get_option('ScanModeHandler','conf.crawl_mode')
    def crawl_mode_dynamic_fuzz_suffix(self) -> str:
        return self._get_option('ScanModeHandler','conf.crawl_mode_dynamic_fuzz_suffix')
    def crawl_mode_parse_robots(self) -> str:
        return self._get_option('ScanModeHandler','conf.crawl_mode_parse_robots')
    def crawl_mode_parse_html(self) -> str:
        return self._get_option('ScanModeHandler','conf.crawl_mode_parse_html')
    def crawl_mode_dynamic_fuzz(self) -> str:
        return self._get_option('ScanModeHandler','conf.crawl_mode_dynamic_fuzz')
    def fuzz_mode(self) -> str:
        return self._get_option('ScanModeHandler','conf.fuzz_mode')
    def fuzz_mode_load_single_dict(self) -> str:
        return self._get_option('ScanModeHandler','conf.fuzz_mode_load_single_dict')
    def fuzz_mode_load_mult_dict(self) -> str:
        return self._get_option('ScanModeHandler','conf.fuzz_mode_load_mult_dict')
    def fuzz_mode_label(self) -> str:
        return self._get_option('ScanModeHandler','conf.fuzz_mode_label')

    def request_headers(self) -> str:
        return self._get_option('RequestHandler','conf.request_headers')
    def request_header_ua(self) -> str:
        return self._get_option('RequestHandler','conf.request_header_ua')
    def request_header_cookie(self) -> str:
        return self._get_option('RequestHandler','conf.request_header_cookie')
    def request_header_401_auth(self) -> str:
        return self._get_option('RequestHandler','conf.request_header_401_auth')
    def request_timeout(self) -> str:
        return self._get_option('RequestHandler','conf.request_timeout')
    def request_delay(self) -> str:
        return self._get_option('RequestHandler','conf.request_delay')
    def request_limit(self) -> str:
        return self._get_option('RequestHandler','conf.request_limit')
    def request_max_retries(self) -> str:
        return self._get_option('RequestHandler','conf.request_max_retries')
    def request_persistent_connect(self) -> str:
        return self._get_option('RequestHandler','conf.request_persistent_connect')
    def request_method(self) -> str:
        return self._get_option('RequestHandler','conf.request_method')
    def redirection_302(self) -> str:
        return self._get_option('RequestHandler','conf.redirection_302')
    def file_extension(self) -> str:
        return self._get_option('RequestHandler','conf.file_extension')

    def response_status_code(self) -> str:
        return self._get_option('ResponseHandler','conf.response_status_code')
    def response_header_content_type(self) -> str:
        return self._get_option('ResponseHandler','conf.response_header_content_type')
    def response_size(self) -> str:
        return self._get_option('ResponseHandler','conf.response_size')
    def auto_check_404_page(self) -> str:
        return self._get_option('ResponseHandler','conf.auto_check_404_page')
    def custom_503_page(self) -> str:
        return self._get_option('ResponseHandler','conf.custom_503_page')
    def custom_response_page(self) -> str:
        return self._get_option('ResponseHandler','conf.custom_response_page')
    def skip_size(self) -> str:
        return self._get_option('ResponseHandler','conf.skip_size')

    def proxy_server(self) -> str:
        return self._get_option('ProxyHandler','conf.proxy_server')

    def debug(self) -> str:
        return self._get_option('DebugMode','conf.debug')
    def update(self) -> str:
        return self._get_option('CheckUpdate','conf.update')



#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: xxlin
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-04-10 17:48:54
'''

import copy
import types
from typing import Any, TypeVar, cast

from collections.abc import Mapping

T = TypeVar('T')
K = TypeVar('K')
V = TypeVar('V')

class AttribDict(dict[K, V]):
    """
    This class defines the project object, inheriting from Python data
    type dictionary.

    >>> foo = AttribDict()
    >>> foo.bar = 1
    >>> foo.bar
    1
    """
    
    attribute: Any
    __initialised: bool

    def __init__(self, indict: Mapping[K, V] | None = None, attribute: Any = None):
        if indict is None:
            indict = cast(Mapping[K, V], {})

        # Set any attributes here - before initialisation
        # these remain as normal attributes
        self.attribute = attribute
        dict.__init__(self, indict)
        self.__initialised = True

        # After initialisation, setting attributes
        # is the same as setting an item

    def __getattr__(self, item: str) -> Any:
        """
        Maps values to attributes
        Only called if there *is NOT* an attribute with this name
        """

        try:
            return self.__getitem__(cast(K, item))
        except KeyError:
            raise AttributeError("unable to access item '%s'" % item)

    def __setattr__(self, item: str, value: Any) -> None:
        """
        Maps attributes to values
        Only if we are initialised
        """

        # This test allows attributes to be set in the __init__ method
        if "_AttribDict__initialised" not in self.__dict__:
            return dict.__setattr__(self, item, value)

        # Any normal attributes are handled normally
        elif item in self.__dict__:
            dict.__setattr__(self, item, value)

        else:
            self.__setitem__(cast(K, item), cast(V, value))

    def __getstate__(self) -> dict[str, Any]:
        return self.__dict__

    def __setstate__(self, dict: dict[str, Any]) -> None:
        self.__dict__ = dict

    def __deepcopy__(self, memo: dict[int, Any]) -> 'AttribDict[K, V]':
        retVal = self.__class__()
        memo[id(self)] = retVal

        for attr in dir(self):
            if not attr.startswith('_'):
                value = getattr(self, attr)
                if not isinstance(value, (types.BuiltinFunctionType, types.FunctionType, types.MethodType)):
                    setattr(retVal, attr, copy.deepcopy(value, memo))

        for key, value in self.items():
            retVal.__setitem__(key, copy.deepcopy(value, memo))

        return retVal




#!/usr/bin/env python
import doctest
import sys
if hasattr(doctest, "testfile"):
    total_failures, total_tests = (0, 0)

    print("=== Test file: README ===")
    failure, tests = doctest.testfile('README', optionflags=doctest.ELLIPSIS)
    total_failures += failure
    total_tests += tests

    print("=== Test file: test.rst ===")
    failure, tests = doctest.testfile('test/test.rst', optionflags=doctest.ELLIPSIS)
    total_failures += failure
    total_tests += tests

    print("=== Test IPy module ===")
    import IPy
    failure, tests = doctest.testmod(IPy)
    total_failures += failure
    total_tests += tests

    print("=== Overall Results ===")
    print("total tests %d, failures %d" % (total_tests, total_failures))
    if total_failures:
        sys.exit(1)
else:
    sys.stderr.write("WARNING: doctest has no function testfile (before Python 2.4), unable to check README\n")




"""Unit test for IPy.py

Further Information might be available at http://c0re.jp/c0de/IPy/

Hacked 2001 by drt@un.bewaff.net
"""

# TODO: unify assert / FilIf usage

import sys
import threading
sys.path.append('.')
sys.path.append('..')

import IPy
import unittest

class parseAddress(unittest.TestCase):
    okValues = [('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210', 338770000845734292534325025077361652240),
                ('FEDCBA9876543210FEDCBA9876543210', 338770000845734292534325025077361652240),
                ('0xFEDCBA9876543210FEDCBA9876543210', 338770000845734292534325025077361652240),
                ('1080:0000:0000:0000:0008:0800:200C:417A', 21932261930451111902915077091070067066),
                ('1080:0:0:0:8:800:200C:417A', 21932261930451111902915077091070067066),
                ('1080:0::8:800:200C:417A', 21932261930451111902915077091070067066),
                ('1080::8:800:200C:417A', 21932261930451111902915077091070067066),
                ('FF01:0:0:0:0:0:0:43', 338958331222012082418099330867817087043),
                ('FF01:0:0::0:0:43', 338958331222012082418099330867817087043),
                ('FF01::43', 338958331222012082418099330867817087043),
                ('0:0:0:0:0:0:0:1', 1),
                ('0:0:0::0:0:1', 1),
                ('::1', 1),
                ('0:0:0:0:0:0:0:0', 0),
                ('0:0:0::0:0:0', 0),
                ('::', 0),
                ('0:0:0:0:0:0:13.1.68.3', 218186755),
                ('::13.1.68.3', 218186755),
                ('0:0:0:0:0:FFFF:129.144.52.38', 281472855454758),
                ('::FFFF:129.144.52.38', 281472855454758),
                ('1080:0:0:0:8:800:200C:417A', 21932261930451111902915077091070067066),
                ('1080::8:800:200C:417A', 21932261930451111902915077091070067066),
                ('0.0.0.0', 0),
                ('0', 0),
                ('127.0.0.1', 2130706433),
                ('255.255.255.255', 4294967295),
                ('0.0.0.1', 1),
                ('1', 16777216),
                ('213.221.113.87', 3588059479),
                ('0000', 0),
                ('127001', 127001),
                ('1234576', 1234576),
                ('1', 16777216),
                ('232111387', 232111387),
                ('255', 4278190080),
                ('256', 256),
                ('0xffffffff', 4294967295),
                ('0x100000000', 4294967296),
                ('0xffffffffffffffffffffffffffffffff', 0xffffffffffffffffffffffffffffffff),
                ('0xdeadbeef', 0xdeadbeef),
                ('0xdeadbabe', 0xdeadbabe),
                ('0xdeadc0de', 0xdeadc0de),
                ('0xc0decafe', 0xc0decafe),
                ('0xc0debabe', 0xc0debabe),
                ('0xbabec0de', 0xbabec0de),
                ('0xcafebabe', 0xcafebabe),
                ('0x1', 1),
                ('0xabcdef', 11259375)]

    # TODO: check for more invalid input

    def testKnownValues(self):
        """parsing of known values should give known results"""
        for x in self.okValues:
            (question, answer) = x
            (result, version) = IPy.parseAddress(question)
            self.assertEqual(answer, result, f"{question!r}, {answer!r}, {result!r}")

    def testVersionDistinction(self):
        """problems destinguishing IPv4 and IPv6"""
        (result, version) = IPy.parseAddress('0xffffffff')
        self.assertEqual(version, 4)
        (result, version) = IPy.parseAddress('0x100000000')
        self.assertEqual(version, 6)

    def testEmpty(self):
        """'' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '')

    def testTooBig(self):
        """'' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '0x100000000000000000000000000000000')

    def testLongIPv4(self):
        """'1.2.3.4.5' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '1.2.3.4.5')

    def testNonByteIPv4(self):
        """'1.2.3.256' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '1.2.3.256')

    def testNegativeByteIPv4(self):
        """'-1.2.3.4' and '1.2.3.-4' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '-1.2.3.4')
        self.assertRaises(ValueError, IPy.parseAddress, '1.2.3.-4')

    def testTripleColonIPv6(self):
        """'2001:::1' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '2001:::1')

    def testRepeatDoubleColonIPv6(self):
        """'2001::ABCD::1' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '2001::ABCD::1')

    def testDoubleColonWithEightHextetsIPv6(self):
        """'1111::2222:3333:4444:5555:6666:7777:8888' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '1111::2222:3333:4444:5555:6666:7777:8888')

    def testBeginningColonWithEightHextetsIPv6(self):
        """':1111:2222:3333:4444:5555:6666:7777:8888' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, ':1111:2222:3333:4444:5555:6666:7777:8888')

    def testEndingColonWithEightHextetsIPv6(self):
        """'1111:2222:3333:4444:5555:6666:7777:8888:' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '1111:2222:3333:4444:5555:6666:7777:8888:')

    def testNegativeHexletIPv6(self):
        """'2001:-ABCD::1' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '2001:-ABCD::1')

    def testTooBigHexletIPv6(self):
        """'2001:10000::1' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '2001:10000::1')

    def testShortAddressIPv6(self):
        """'1111:2222:3333:4444:5555:6666:7777' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '1111:2222:3333:4444:5555:6666:7777')

    def testLongAddressIPv6(self):
        """'1111:2222:3333:4444:5555:6666:7777:8888:9999' should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, '1111:2222:3333:4444:5555:6666:7777:8888:9999')

    def testBogusValues(self):
        """Text values should raise an exception"""
        self.assertRaises(ValueError, IPy.parseAddress, 'xx')
        self.assertRaises(ValueError, IPy.parseAddress, 'foobar')

class _intToIP(unittest.TestCase):
    v4values = [(0x7f000001, '127.0.0.1'),
                (0x0, '0.0.0.0'),
                (0x1, '0.0.0.1'),
                (0xf, '0.0.0.15'),
                (0xff, '0.0.0.255'),
                (0xFFFFFFFF, '255.255.255.255')]
    v6values = [(0x7f000001, '0000:0000:0000:0000:0000:0000:7f00:0001'),
                (0x0, '0000:0000:0000:0000:0000:0000:0000:0000'),
                (0x1, '0000:0000:0000:0000:0000:0000:0000:0001'),
                (0xf, '0000:0000:0000:0000:0000:0000:0000:000f'),
                (0xff, '0000:0000:0000:0000:0000:0000:0000:00ff'),
                (0xFFFFFFFF, '0000:0000:0000:0000:0000:0000:ffff:ffff'),
                (0x100000000, '0000:0000:0000:0000:0000:0001:0000:0000'),
                (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF, 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')]

    def testKnownValuesv4(self):
        """printing of known IPv4 values should give known results"""
        for x in self.v4values:
            (question, answer) = x
            result  = IPy.intToIp(question, 4).lower()
            self.assertEqual(answer, result, f"{question!r}, {answer!r}, {result!r}")

    def testKnownValuesv6(self):
        """printing of known IPv6 values should give known results"""
        for x in self.v6values:
            (question, answer) = x
            result  = IPy.intToIp(question, 6).lower()
            self.assertEqual(answer, result, f"{question!r}, {answer!r}, {result!r}")

    def testNegativeIPv4(self):
        """negative IPv4 Values should raise an exception"""
        self.assertRaises(ValueError, IPy.intToIp, -1, 4)

    def testNegativeIPv6(self):
        """negative IPv6 Values should raise an exception"""
        self.assertRaises(ValueError, IPy.intToIp, -1, 6)

    def testLargeIPv4(self):
        """IPv4: Values > 0xffffffff should raise an exception"""
        self.assertRaises(ValueError, IPy.intToIp, 0x100000000, 4)

    def testLargeIPv6(self):
        """IPv6: Values > 0xffffffffffffffffffffffffffffffff should raise an exception"""
        self.assertRaises(ValueError, IPy.intToIp, 0x100000000000000000000000000000000, 6)

    def testIllegalVersion(self):
        """IPVersion other than 4 and 6 should raise an exception"""
        self.assertRaises(ValueError, IPy.intToIp, 1, 0)
        self.assertRaises(ValueError, IPy.intToIp, 1, 1)
        self.assertRaises(ValueError, IPy.intToIp, 1, 2)
        self.assertRaises(ValueError, IPy.intToIp, 1, 3)
        self.assertRaises(ValueError, IPy.intToIp, 1, 5)
        self.assertRaises(ValueError, IPy.intToIp, 1, 7)
        self.assertRaises(ValueError, IPy.intToIp, 1, 8)

class _countXBits(unittest.TestCase):
    def testCount1Bits(self):
        self.assertEqual(IPy._count1Bits(0), 0)
        self.assertEqual(IPy._count1Bits(0xf), 4)
        self.assertEqual(IPy._count1Bits(0x10), 5)
        self.assertEqual(IPy._count1Bits(0xff), 8)
        self.assertEqual(IPy._count1Bits(0xffff), 16)
        self.assertEqual(IPy._count1Bits(0xffffffff), 32)
        self.assertEqual(IPy._count1Bits(0xffffffffffffffffffffffffffffffff), 128)

    def testCount1Bits(self):
        self.assertEqual(IPy._count0Bits(0), 0)
        self.assertEqual(IPy._count0Bits(0xf0), 4)
        self.assertEqual(IPy._count0Bits(0xf00), 8)
        self.assertEqual(IPy._count0Bits(0xf000), 12)
        self.assertEqual(IPy._count0Bits(0xf0000), 16)
        self.assertEqual(IPy._count0Bits(0xf00000), 20)
        self.assertEqual(IPy._count0Bits(0xf000000), 24)
        self.assertEqual(IPy._count0Bits(0xf0000000), 28)
        self.assertEqual(IPy._count0Bits(0xff000000), 24)
        self.assertEqual(IPy._count0Bits(0xfff00000), 20)
        self.assertEqual(IPy._count0Bits(0x80000000), 31)
        self.assertEqual(IPy._count0Bits(0xf0000000000000000000000000000000), 124)
        self.assertEqual(IPy._count0Bits(0x80000000000000000000000000000000), 127)


class _intToBin(unittest.TestCase):
    knownValues = [(0, '0'), (1, '1'), (2, '10'), (3, '11'), (4, '100'), (5, '101'),
                   (6, '110'), (7, '111'), (8, '1000'), (9, '1001'),
                   (0xf, '1111'), (0xff, '11111111'),
                   (0xFFFFFFFF, '11111111111111111111111111111111'),
                   (0x100000000, '100000000000000000000000000000000'),
                   (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF, '11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'),
                   (0x100000000000000000000000000000000, '100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000')]

    def testKnownValues(self):
        """conversion of known values values should give known results"""
        for x in self.knownValues:
            (question, answer) = x
            result  = IPy._intToBin(question)
            self.assertEqual(answer, result, str(question))

    def testNegativeIPv4(self):
        """negative Values should raise an exception"""
        self.assertRaises(ValueError, IPy._intToBin, -1)

class netmaskPrefixlenConv(unittest.TestCase):
    known4Values = [(0xFFFFFFFF, 32), (0xFFFFFFFE, 31), (0xFFFFFFFC, 30), (0xFFFFFFF8, 29),
                    (0xFFFFFFF0, 28), (0xFFFFFFE0, 27), (0xFFFFFFC0, 26), (0xFFFFFF80, 25),
                    (0xFFFFFF00, 24), (0xFFFFFE00, 23), (0xFFFFFC00, 22), (0xFFFFF800, 21),
                    (0xFFFFF000, 20), (0xFFFFE000, 19), (0xFFFFC000, 18), (0xFFFF8000, 17),
                    (0xFFFF0000, 16), (0xFFFE0000, 15), (0xFFFC0000, 14), (0xFFF80000, 13),
                    (0xFFF00000, 12), (0xFFE00000, 11), (0xFFC00000, 10), (0xFF800000, 9),
                    (0xFF000000, 8), (0xFE000000, 7), (0xFC000000, 6), (0xF8000000, 5),
                    (0xF0000000, 4), (0xE0000000, 3), (0xC0000000, 2), (0x80000000, 1)]
    known6Values = [(0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF, 128),
                    (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE, 127),
                    (0xFFFFFFFFFFFFFFFFFFFFFFFF80000000, 97),
                    (0xFFFFFFFFFFFFFFFFFFFFFFFF00000000, 96),
                    (0xFFFFFFFFFFFFFFFFFFFFFFFE00000000, 95),
                    (0xFFFFFFFFFFFFFFFF8000000000000000, 65),
                    (0xFFFFFFFFFFFFFFFF0000000000000000, 64),
                    (0xFFFFFFFFFFFFFFFE0000000000000000, 63),
                    (0xFFFFFFFF800000000000000000000000, 33),
                    (0xFFFFFFFF000000000000000000000000, 32),
                    (0xFFFFFFFE000000000000000000000000, 31),
                    (0xC0000000000000000000000000000000, 2),
                    (0x80000000000000000000000000000000, 1)]

    def testKnownValuesv4n2p(self):
        """conversion of known values values should give known results"""
        for x in self.known4Values:
            (question, answer) = x
            result  = IPy._netmaskToPrefixlen(question)
            self.assertEqual(answer, result, hex(question))

    def testKnownValuesv6n2p(self):
        """conversion of known values values should give known results"""
        for x in self.known6Values:
            (question, answer) = x
            result  = IPy._netmaskToPrefixlen(question)
            self.assertEqual(answer, result, hex(question))

    def testKnownValuesv4p2n(self):
        """conversion of known values values should give known results"""
        for x in self.known4Values:
            (answer, question) = x
            result  = IPy._prefixlenToNetmask(question, 4)
            self.assertEqual(answer, result, hex(question))

    def testKnownValuesv6p2n(self):
        """conversion of known values values should give known results"""
        for x in self.known6Values:
            (answer, question) = x
            result  = IPy._prefixlenToNetmask(question, 6)
            self.assertEqual(answer, result, "%d: %s != %s" % (question, hex(answer), result))

    def testInvalidv4n2p(self):
        """Netmasks should be all ones in the first part and all zeros in the second part"""
        self.assertRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00)

    def testInvalidv6n2p(self):
        """Netmasks should be all ones in the first part and all zeros in the second part"""
        self.assertRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00ff00ff00ff00ff00ff00ff00)


class checkChecks(unittest.TestCase):

    def testCheckNetmaskOk(self):
        """Legal Netmasks should be allowed."""
        self.assertFalse(IPy._checkNetmask(0xffffffff, 32))
        self.assertFalse(IPy._checkNetmask(0xffffff00, 32))
        self.assertFalse(IPy._checkNetmask(0xffff0000, 32))
        self.assertFalse(IPy._checkNetmask(0xff000000, 32))
        self.assertFalse(IPy._checkNetmask(0, 32))

    def testCheckNetmaskFail(self):
        """Illegal Netmasks should be rejected."""
        self.assertRaises(ValueError, IPy._checkNetmask, 0xf0ffffff, 32)
        self.assertRaises(ValueError, IPy._checkNetmask, 0xf0f0f0f0, 32)
        self.assertRaises(ValueError, IPy._checkNetmask, 0xff00ff00, 32)
        self.assertRaises(ValueError, IPy._checkNetmask, 0x70000001, 32)
        self.assertRaises(ValueError, IPy._checkNetmask, 0xfffffff, 32)

    def testCheckPrefixOk(self):
        """Legal IP/prefix combinations should check ok."""
        self.assertTrue(IPy._checkPrefix(0x0, 32, 4))
        self.assertTrue(IPy._checkPrefix(0xffffffff, 32, 4))
        self.assertTrue(IPy._checkPrefix(0x7f000001, 32, 4))
        self.assertTrue(IPy._checkPrefix(0x80000000, 1, 4))
        self.assertTrue(IPy._checkPrefix(0x40000000, 2, 4))
        self.assertTrue(IPy._checkPrefix(0x80000000, 3, 4))
        self.assertTrue(IPy._checkPrefix(0x80000000, 4, 4))
        self.assertTrue(IPy._checkPrefix(0xffffff00, 24, 4))
        self.assertTrue(IPy._checkPrefix(0xffffff00, 24, 4))
        self.assertTrue(IPy._checkPrefix(0xfffffff0, 28, 4))
        self.assertTrue(IPy._checkPrefix(0x0, 32, 4))
        self.assertTrue(IPy._checkPrefix(0x0, 1, 4))
        self.assertTrue(IPy._checkPrefix(0x0, 0, 4))
        self.assertTrue(IPy._checkPrefix(0xffffffffffffffff0000000000000000, 64, 6))
        self.assertTrue(IPy._checkPrefix(0x0, 64, 6))
        self.assertTrue(IPy._checkPrefix(0x0, 0, 6))
        self.assertTrue(IPy._checkPrefix(0x0, 128, 6))
        self.assertTrue(IPy._checkPrefix(0xffffffffffffffffffffffffffffffff, 128, 6))


    def testCheckPrefixFail(self):
        """Illegal Prefixes should be catched."""
        self.assertFalse(IPy._checkPrefix(0x7f000001, -1, 4))
        self.assertFalse(IPy._checkPrefix(0x7f000001, 33, 4))
        self.assertFalse(IPy._checkPrefix(0x7f000001, 24, 4))
        self.assertFalse(IPy._checkPrefix(0x7f000001, 31, 4))
        self.assertFalse(IPy._checkPrefix(0x7f000080, 24, 4))
        self.assertFalse(IPy._checkPrefix(0x7f000100, 23, 4))
        self.assertFalse(IPy._checkPrefix(0x7f000000, 1, 4))
        self.assertFalse(IPy._checkPrefix(0x7f000000, 0, 4))
        self.assertFalse(IPy._checkPrefix(0x1, -1, 6))
        self.assertFalse(IPy._checkPrefix(0x1, 129, 6))
        self.assertFalse(IPy._checkPrefix(0xffffffffffffffff0000000000000001, 64, 6))
        self.assertFalse(IPy._checkPrefix(0xffffffffffffffff1000000000000000, 64, 6))


    # TODO: _checkNetaddrWorksWithPrefixlen(net, prefixlen, version):

class PythonObjectBehaviour(unittest.TestCase):
    def testIfUsuableAsDictionaryKey(self):
        """IP Object should be usable as dictionary key"""
        d = {}
        d[IPy.IP('127.0.0.1')] = 1
        d[IPy.IP('2001::1')] = 1
        d[IPy.IP('127.0.0.0/24')] = 1
        d[IPy.IP('2001::/64')] = 1

    def testIfCanBeInteratedOver(self):
        """It should be possible to iterate over an IP Object."""
        i = 0
        for x in IPy.IP('127.0.0.0/24'):
            i += 1
        self.assertEqual(i, 256, "iteration over a /24 should yiels 256 values")
        i = 0
        for x in IPy.IP('2001::/124'):
            i += 1
        self.assertEqual(i, 16, "iteration over a /124 should yiels 16 values")

    def testIfComparesEqual(self):
        """nets of the same base and size should be considered equal, others not"""
        a = IPy.IP('127.0.0.0/24')
        a2 = a
        b = IPy.IP('127.0.0.0/24')
        c = IPy.IP('127.0.0.0/23')
        d = IPy.IP('127.0.0.0/22')
        e = IPy.IP('64.0.0.0/24')
        self.assertEqual(a2, a)
        self.assertEqual(a2, b)
        self.assertEqual(a, a)
        self.assertEqual(a, b)
        self.assertNotEqual(a, c)
        self.assertNotEqual(a, d)
        self.assertNotEqual(a, e)
        self.assertNotEqual(b, c)
        self.assertNotEqual(b, d)
        self.assertNotEqual(b, e)
        self.assertNotEqual(c, d)
        self.assertNotEqual(c, e)
        self.assertNotEqual(d, e)

    def testIfContainsInt(self):
        """__contains__() should work somewhat with ints"""
        ip = IPy.IP('127.0.0.0/28')
        for x in ip:
            self.assertTrue(x.int() in ip)
        ip = IPy.IP('2001::/124')
        for x in ip:
            self.assertTrue(x.int() in ip)

    def testIfContainsStr(self):
        """__contains__() should work somewhat with strings"""
        ip = IPy.IP('127.0.0.0/28')
        for x in ip:
            self.assertTrue(x.strNormal() in ip, f"{x.strNormal()!r} not in {ip!r}")
        ip = IPy.IP('2001::/124')
        for x in ip:
            self.assertTrue(x.strNormal() in ip, f"{x.strNormal()!r} not in {ip!r}")

    def testIfContainsIPobj(self):
        """__contains__() should work somewhat with IP instances"""
        ip = IPy.IP('127.0.0.0/28')
        for x in ip:
            self.assertTrue(x in ip)
        ip = IPy.IP('2001::/124')
        for x in ip:
            self.assertTrue(x in ip)

    def testContainsVersionSeparation(self):
        """__contains__() should return false if versions mismatch"""
        four = IPy.IP('192.168.0.0/16')
        six = IPy.IP('::c0a8:0/112')
        self.assertFalse(four in six)
        self.assertFalse(six in four)

    def testActingAsArray(self):
        """An IP-object should handle indices."""
        ip = IPy.IP('127.0.0.0/24')
        self.assertEqual(ip[0], ip.net())
        self.assertEqual(ip[-1], ip.broadcast())
        self.assertTrue(ip[255])
        self.assertTrue(isinstance(ip[4::4], list))
        self.assertRaises(IndexError, ip.__getitem__, 256)

    def testStr(self):
        """string() should work somewhat with IP instances"""
        ip = IPy.IP('127.0.0.0/28')
        for x in ip:
            self.assertTrue(str(x))
        ip = IPy.IP('2001::/124')
        for x in ip:
            self.assertTrue(str(x))

    def testRepr(self):
        """repr() should work somewhat with IP instances"""
        ip = IPy.IP('127.0.0.0/28')
        for x in ip:
            self.assertTrue(repr(x))
        ip = IPy.IP('2001::/124')
        for x in ip:
            self.assertTrue(repr(x))

    def testLen(self):
        """object should have an working __len__() interface."""
        self.assertEqual(len(IPy.IP('127.0.0.0/28')), 16)
        self.assertEqual(len(IPy.IP('127.0.0.0/30')), 4)
        self.assertEqual(len(IPy.IP('127.0.0.0/26')), 64)
        self.assertEqual(len(IPy.IP('127.0.0.0/16')), 2**16)

    # cmp
    # IP[0xffffffff]
    # IP + IP
    # reverse
    # netmsk
    # ip

class IPobject(unittest.TestCase):
    def testStrCompressed(self):
        """Compressed string Output."""
        testValues = ['127.0.0.1',
                  'dead::beef',
                  'dead:beef::',
                  'dead:beef::/48',
                  'ff00:1::',
                  'ff00:0:f000::',
                  '0:0:1000::',
                  '::e000:0/112',
                  '::e001:0/112',
                  'dead:beef::/48',
                  'ff00:1::/64',
                  'ff00:0:f000::/64',
                  '0:0:1000::/64',
                  '::e000:0/112',
                  '::e001:0/112',
                  '::1:0:0:0:2',
                  '0:1:2:3:4:5:6:7',
                  '1:2:3:4:0:5:6:7',
                  '1:2:3:4:5:6:7:0',
                  '1:0:0:2::',
                  '1:0:0:2::3',
                  '1::2:0:0:3']
        for question in testValues:
            result = IPy.IP(question).strCompressed()
            self.assertEqual(question, result, (question, result))

    def testStrBin(self):
        """Binary string Output."""

        testValues = [('0.0.0.0', '00000000000000000000000000000000'),
                      ('0.0.0.1', '00000000000000000000000000000001'),
                      ('255.255.255.255', '11111111111111111111111111111111'),
                      ('128.0.0.0', '10000000000000000000000000000000'),
                      ('::0', '00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000'),
                      ('::1', '00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001'),
                      ('ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff', '11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'),
                      ('5555:5555:5555:5555:5555:5555:5555:5555', '01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101'),
                      ('aaaa:aaaa:aaaa:aaaa:aaaa:aaaa:aaaa:aaaa', '10101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010'),
                      ('85.85.85.85', '01010101010101010101010101010101'),
                      ('170.170.170.170', '10101010101010101010101010101010'),
                      ('127.0.0.1', '01111111000000000000000000000001'),
                      ('1::2:0:0:3', '00000000000000010000000000000000000000000000000000000000000000000000000000000010000000000000000000000000000000000000000000000011')]
        for (question, answer) in testValues:
            result = IPy.IP(question).strBin()
            self.assertEqual(answer, result, (question, answer, result))

    def testStrNormal(self):
        """Normal string Output."""
        testValues = [(338770000845734292534325025077361652240, 'fedc:ba98:7654:3210:fedc:ba98:7654:3210'),
                      (21932261930451111902915077091070067066, '1080:0:0:0:8:800:200c:417a'),
                      (338958331222012082418099330867817087043, 'ff01:0:0:0:0:0:0:43'),
                      (0, '0.0.0.0'),
                      (2130706433, '127.0.0.1'),
                      (4294967295, '255.255.255.255'),
                      (1, '0.0.0.1'),
                      (3588059479, '213.221.113.87')]
        for (question, answer) in testValues:
            result = IPy.IP(question).strNormal(question)
            self.assertEqual(answer, result, (question, result, answer))

    def testStrFullsize(self):
        """Normal / 0-padded string Output."""
        testValues = [(338770000845734292534325025077361652240, 'fedc:ba98:7654:3210:fedc:ba98:7654:3210'),
                      (21932261930451111902915077091070067066, '1080:0000:0000:0000:0008:0800:200c:417a'),
                      (338958331222012082418099330867817087043, 'ff01:0000:0000:0000:0000:0000:0000:0043'),
                      (0, '0.0.0.0'),
                      (2130706433, '127.0.0.1'),
                      (4294967295, '255.255.255.255'),
                      (1, '0.0.0.1'),
                      (3588059479, '213.221.113.87')]
        for (question, answer) in testValues:
            result = IPy.IP(question).strFullsize(question)
            self.assertEqual(answer, result, (question, result, answer))

    def testStrHex(self):
        """Hex string Output."""
        testValues = [(338770000845734292534325025077361652240, '0xfedcba9876543210fedcba9876543210'),
                      (21932261930451111902915077091070067066, '0x108000000000000000080800200c417a'),
                      (338958331222012082418099330867817087043, '0xff010000000000000000000000000043'),
                      (0, '0x0'),
                      (1, '0x1'),
                      (4294967295, '0xffffffff'),
                      (3588059479, '0xd5dd7157'),
                      (0x12345678, '0x12345678')]
        for (question, answer) in testValues:
            result = IPy.IP(question).strHex(question).lower()
            self.assertEqual(answer, result, (question, result, answer))

    def testStrDec(self):
        """Decimal string Output."""
        testValues = [(338770000845734292534325025077361652240, '338770000845734292534325025077361652240'),
                      (21932261930451111902915077091070067066, '21932261930451111902915077091070067066'),
                      (338958331222012082418099330867817087043, '338958331222012082418099330867817087043'),
                      (0, '0'),
                      (1, '1'),
                      (0xFFFFFFFF, '4294967295'),
                      (0xD5DD7157, '3588059479')]
        for (question, answer) in testValues:
            result = IPy.IP(question).strDec(question)
            self.assertEqual(answer, result, (question, result, answer))

    def testNet(self):
        """Returning of the Network Address"""
        self.assertEqual(str(IPy.IP("127.0.0.1").net()), "127.0.0.1")
        self.assertEqual(str(IPy.IP("0.0.0.0/0").net()), "0.0.0.0")
        self.assertEqual(str(IPy.IP("2001:1234:5678:1234::/64").net()), "2001:1234:5678:1234::")


    def testBroadcast(self):
        """Returning of broadcast address."""
        self.assertEqual(str(IPy.IP("127.0.0.1").broadcast()), "127.0.0.1")
        self.assertEqual(str(IPy.IP("0.0.0.0/0").broadcast()), "255.255.255.255")
        self.assertEqual(str(IPy.IP("2001:1234:5678:1234::/64").broadcast()), "2001:1234:5678:1234:ffff:ffff:ffff:ffff")


    def testStrNetmask(self):
        """StrNetmask should return netmasks"""
        self.assertEqual(IPy.IP("0.0.0.0/0").strNetmask(), "0.0.0.0")
        self.assertEqual(IPy.IP("0.0.0.0/32").strNetmask(), "255.255.255.255")
        self.assertEqual(IPy.IP("127.0.0.0/24").strNetmask(), "255.255.255.0")
        self.assertEqual(IPy.IP("2001:1234:5678:1234::/64").strNetmask(), "/64")


    def testNetmask(self):
        """Netmask should return netmasks"""
        self.assertEqual(str(IPy.IP("0.0.0.0/0").netmask()), "0.0.0.0")
        self.assertEqual(str(IPy.IP("0.0.0.0/32").netmask()), "255.255.255.255")
        self.assertEqual(str(IPy.IP("127.0.0.0/24").netmask()), "255.255.255.0")
        self.assertEqual(str(IPy.IP("2001:1234:5678:1234::/64").netmask()), "ffff:ffff:ffff:ffff:0000:0000:0000:0000")

    def testInt(self):
        """Prefixlen"""
        self.assertEqual(IPy.IP("127.0.0.1").int(), 2130706433)
        self.assertEqual(IPy.IP("0.0.0.0").int(), 0)
        self.assertEqual(IPy.IP("255.255.255.255").int(), 0xffffffff)
        self.assertEqual(IPy.IP("0000:0000:0000:0000:0000:0000:0000:0000").int(), 0)
        self.assertEqual(IPy.IP("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff").int(), 0xffffffffffffffffffffffffffffffff)
        self.assertEqual(IPy.IP("2001:1234:5678:9abc:de00:0000:0000:0000").int(), 42540857391974671903776007410583339008)


    def testPrefixlen(self):
        """Prefixlen"""
        self.assertEqual(IPy.IP("127.0.0.1").prefixlen(), 32)
        self.assertEqual(IPy.IP("::1").prefixlen(), 128)
        self.assertEqual(IPy.IP("10.0.0.0/24").prefixlen(), 24)
        self.assertEqual(IPy.IP("10.0.0.0-10.0.0.255").prefixlen(), 24)
        self.assertEqual(IPy.IP("10.0.0.0/255.255.255.0").prefixlen(), 24)
        self.assertEqual(IPy.IP("2001::/64").prefixlen(), 64)


    def testVersion(self):
        """IP-version detection should work"""
        self.assertEqual(IPy.IP("0.0.0.0/0").version(), 4)
        self.assertEqual(IPy.IP("::1").version(), 6)

    # TODO:
    #def reverseNames(self):
    #def reverseName(self):
    #def __cmp__(self, other):
    #def __add__(self, other):
    #def _printPrefix(self, want):

    def testOverlaps(self):
        """Overlapping Address Ranges."""
        testValues = [('192.168.0.0/23', '192.168.1.0/24', 1),
                      ('192.168.0.0/23', '192.168.0.0/20', 1),
                      ('192.168.0.0/23', '192.168.2.0', 0),
                      ('192.168.0.0/23', '192.167.255.255', 0),
                      ('192.168.0.0/23', '192.168.0.0', 1),
                      ('192.168.0.0/23', '192.168.1.255', 1),
                      ('192.168.1.0/24', '192.168.0.0/23', -1),
                      ('127.0.0.1', '127.0.0.1', 1),
                      ('127.0.0.1', '127.0.0.2', 0)]
        for (a, b, answer) in testValues:
            result = IPy.IP(a).overlaps(b)
            self.assertEqual(answer, result, (a, b, result, answer))

    def testNetmask(self):
        """Normal string Output."""
        testValues = [(338770000845734292534325025077361652240, '0xfedcba9876543210fedcba9876543210'),
                      (21932261930451111902915077091070067066, '0x108000000000000000080800200c417a'),
                      (338958331222012082418099330867817087043, '0xff010000000000000000000000000043'),
                      (0, '0x0'),
                      (1, '0x1'),
                      (4294967295, '0xffffffff'),
                      (3588059479, '0xd5dd7157')]
        for (question, answer) in testValues:
            result = IPy.IP(question).strHex(question).lower()
            self.assertEqual(answer, result, (question, result, answer))

    def testV46map(self):
        four    = IPy.IP('192.168.1.1')
        six     = IPy.IP('::ffff:192.168.1.1')
        invalid = IPy.IP('2001::ffff:192.168.1.1')
        self.assertEqual(four.v46map(), six)
        self.assertEqual(four, six.v46map())
        self.assertRaises(ValueError, invalid.v46map)

# TODO
#eval(repr(IPy))
# differences between IP and IPint


# I ported this checks to be sure that I don't have errors in my own checks.
class NetIPChecks(unittest.TestCase):
    """Checks taken from perls Net::IP"""
    def testMisc(self):
        ip = IPy.IP('195.114.80/24')
        self.assertEqual(ip.int(), 3279048704)
        self.assertEqual(ip.reverseName(),'80.114.195.in-addr.arpa.')
        self.assertEqual(ip.strBin(),'11000011011100100101000000000000')
        self.assertEqual(str(ip.net()),'195.114.80.0')
        self.assertEqual(str(ip),'195.114.80.0/24')
        self.assertEqual(ip.prefixlen(),24)
        self.assertEqual(ip.version(),4)
        self.assertEqual(ip.len(),256)
        self.assertEqual(IPy._intToBin(ip.netmask().int()),'11111111111111111111111100000000')
        self.assertEqual(ip.strNetmask(),'255.255.255.0')
        self.assertEqual(ip.iptype(), 'PUBLIC')
        self.assertEqual(ip.broadcast().strBin(),'11000011011100100101000011111111')
        self.assertEqual(str(ip.broadcast()),'195.114.80.255')

        ip = IPy.IP('202.31.4/24')
        self.assertEqual(str(ip.net()),'202.31.4.0')

        self.assertRaises(ValueError, IPy.IP, '234.245.252.253/2')

        # because we ar using integer representation we don't need a special "binadd"
        ip = IPy.IP('62.33.41.9')
        ip2 = IPy.IP('0.1.0.5')
        self.assertEqual(str(IPy.IP(ip.int() + ip2.int())),'62.34.41.14')
        #$T->ok_eq ($ip->binadd($ip2)->ip(),'62.34.41.14',$ip->error());

        ip = IPy.IP('133.45.0/24')
        ip2 = IPy.IP('133.45.1/24')
        ip3 = IPy.IP('133.45.2/24')
        self.assertEqual((ip + ip2).prefixlen(),23)
        # Non-adjacent ranges
        self.assertRaises(ValueError, IPy.IP.__add__, ip, ip3)
        # Resulting invalid prefix
        self.assertRaises(ValueError, IPy.IP.__add__, ip2, ip3)

        ip2 = IPy.IP('133.44.255.255')
        #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());

        # this is something we can't do with IPy
        #ip = IPy.IP('133.44.255.255-133.45.0.42');
        #$T->ok_eq (($ip->find_prefixes())[3],'133.45.0.40/31',$ip->error());

        ip = IPy.IP('201.33.128.0/22')
        ip2 = IPy.IP('201.33.129.0/24')
        #$T->ok_eqnum ($ip->overlaps($ip2),$IP_B_IN_A_OVERLAP,$ip->error());

        ip = IPy.IP('dead:beef:0::/48')
        self.assertEqual(str(ip.net()),'dead:beef::')
        self.assertEqual(ip.int(), 295990755014133383690938178081940045824)
        self.assertEqual(ip.strBin(),'11011110101011011011111011101111000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000')
        self.assertEqual(ip.strCompressed(),'dead:beef::/48')
        self.assertEqual(ip.prefixlen(), 48)
        self.assertEqual(ip.version(), 6)
        self.assertEqual(ip.strNetmask(),'/48')
        self.assertEqual(str(ip.netmask()),'ffff:ffff:ffff::')
        self.assertEqual(ip.iptype(),'RESERVED')
        self.assertEqual(ip.reverseName(),'0.0.0.0.f.e.e.b.d.a.e.d.ip6.arpa.')
        self.assertEqual(str(ip.broadcast()),'dead:beef:0:ffff:ffff:ffff:ffff:ffff')

        ip = IPy.IP('202.31.4/24')
        self.assertEqual(str(ip.net()),'202.31.4.0')

        # TODO: fix this in IPy ... after rereading the RfC
        # ip = IPy.IP(':1/128');
        #$T->ok_eq ($ip->error(),'Invalid address :1 (starts with :)',$ip->error());
        #$T->ok_eqnum ($ip->errno(),109,$ip->error());

        ip = IPy.IP('ff00:0:f000::')
        ip2 = IPy.IP('0:0:1000::')
        self.assertEqual(IPy.IP(ip.int() + ip2.int()).strCompressed(), 'ff00:1::')

        ip = IPy.IP('::e000:0/112')
        ip2 = IPy.IP('::e001:0/112')
        self.assertEqual(ip.__add__(ip2).prefixlen(),111)
        self.assertEqual(ip.__add__(ip2).version(),6)

        ip2 = IPy.IP('::dfff:ffff')
        #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());

        #ip = IPy.IP('::e000:0 - ::e002:42')
        #$T->ok_eq (($ip->find_prefixes())[2],'0000:0000:0000:0000:0000:0000:e002:0040/127',$ip->error());

        ip = IPy.IP('ffff::/16')
        ip2 = IPy.IP('8000::/16')
        #$T->ok_eqnum ($ip->overlaps($ip2),$IP_NO_OVERLAP,$ip->error());

def timeout(func, args=(), kwargs={}, timeout_duration=1, default=None):
    """
    ASPN receipe written by dustin lee to call a function with
    a timeout using threads:
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/473878

    Small patch: add setDaemon(True) to allow Python to leave whereas the
    thread is not done.
    """
    class InterruptableThread(threading.Thread):
        def __init__(self):
            threading.Thread.__init__(self)
            self.result = None

        def run(self):
            try:
                self.result = func(*args, **kwargs)
            except:
                self.result = default

    it = InterruptableThread()
    it.setDaemon(True)
    it.start()
    it.join(timeout_duration)
    if it.isAlive():
        return default
    else:
        return it.result

class IPSetChecks(unittest.TestCase):
    def setUp(self):
        #array
        self.a = [IPy.IP("192.168." + str(i) + ".0/24") for i in range(256)]
        #range
        self.r = IPy.IP('192.168.0.0/16')
        #testing set
        self.t = IPy.IPSet(self.a)
        #control set
        self.c = IPy.IPSet(self.a)
        #Could otherwise look like 192.168.128.0/17
        self.sixRange = IPy.IP('::c0a8:8000/113')

    def testVersionSeparation(self):
        #Don't remove a matching IPv6 subnet from an IPv4 list
        self.assertRaises(KeyError, self.t.remove, self.sixRange)
        self.t.add(self.sixRange)
        self.assertNotEqual(self.t, self.c)
        self.t.remove(self.sixRange)
        self.t.discard(self.sixRange)
        self.assertEqual(self.t, self.c)

    def testAnd(self):
        ten24s = IPy.IPSet([
            IPy.IP('10.0.1.0/24'),
            IPy.IP('10.0.3.0/24'),
            IPy.IP('10.0.5.0/24'),
            IPy.IP('10.0.7.0/24'),
        ])

        self.assertEqual(ten24s & IPy.IPSet([IPy.IP('10.0.1.10')]),
                         IPy.IPSet([IPy.IP('10.0.1.10')]))

        self.assertEqual(ten24s & IPy.IPSet([
            IPy.IP('10.0.0.99'),
            IPy.IP('10.0.1.10'),
            IPy.IP('10.0.3.40'),
            IPy.IP('11.1.1.99'),
        ]), IPy.IPSet([
            IPy.IP('10.0.1.10'),
            IPy.IP('10.0.3.40'),
        ]))

    def testContains(self):
        self.assertTrue(IPy.IP('192.168.15.32/28') in self.t)
        self.assertFalse(IPy.IP('192.169.15.32/28') in self.t)

        # test for a regression where __contains__ prematurely returns False
        # after testing a prefix length where all IP instances are greater than
        # the query IP.
        ipset = IPy.IPSet([IPy.IP('10.0.0.0/8'), IPy.IP('128.0.0.0/1')])
        self.assertTrue(IPy.IP('10.0.0.0') in ipset)

    def testIsdisjoint(self):
        self.assertTrue(IPy.IPSet([IPy.IP('0.0.0.0/1')])
                .isdisjoint(IPy.IPSet([IPy.IP('128.0.0.0/1')])))
        self.assertFalse(IPy.IPSet([IPy.IP('0.0.0.0/1')])
                .isdisjoint(IPy.IPSet([IPy.IP('0.0.0.0/2')])))
        self.assertFalse(IPy.IPSet([IPy.IP('0.0.0.0/2')])
                .isdisjoint(IPy.IPSet([IPy.IP('0.0.0.0/1')])))
        self.assertFalse(IPy.IPSet([IPy.IP('0.0.0.0/2')])
                .isdisjoint(IPy.IPSet([IPy.IP('0.1.2.3')])))
        self.assertFalse(IPy.IPSet([IPy.IP('0.1.2.3')])
                .isdisjoint(IPy.IPSet([IPy.IP('0.0.0.0/2')])))
        self.assertTrue(IPy.IPSet([IPy.IP('1.1.1.1'), IPy.IP('1.1.1.3')])
                .isdisjoint(IPy.IPSet([IPy.IP('1.1.1.2'), IPy.IP('1.1.1.4')])))
        self.assertFalse(IPy.IPSet([IPy.IP('1.1.1.1'), IPy.IP('1.1.1.3'), IPy.IP('1.1.2.0/24')])
                .isdisjoint(IPy.IPSet([IPy.IP('1.1.2.2'), IPy.IP('1.1.1.4')])))

class RegressionTest(unittest.TestCase):
    def testNulNetmask(self):
        ip = timeout(IPy.IP, ["0.0.0.0/0.0.0.0"], timeout_duration=0.250, default=None)
        if ip:
            text = str(ip)
        else:
            text = "*TIMEOUT*"
        self.assertEqual(text, "0.0.0.0/0")

    def testNonZeroType(self):
        self.assertEqual(bool(IPy.IP("0.0.0.0/0")), True)

    def testPrivate169(self):
        """
        RFC 3330 indicates that 169.254.0.0/16 addresses are private.
        They are automatically configured for links in the absence of other
        information and should not be used on the internet
        """
        self.assertEqual(IPy.IP("169.254.191.164").iptype(), "PRIVATE")

    def testCheckAddrPrefixlenOn(self):
        self.assertEqual(len(IPy.IP('192.168.0.0/24')), 256)
        self.assertRaises(ValueError, IPy.IP, '192.168.1.0/42')
        self.assertRaises(ValueError, IPy.IP, '172.30.1.0/22')

    def testCheckAddrPrefixlenOff(self):
        self.assertEqual(len(IPy.IP('192.168.0.0/24')), 256)
        self.assertRaises(ValueError, IPy.IP, '192.168.1.0/42')

class TestConstrutor(unittest.TestCase):
    def testCheckAddrPrefixlenOff(self):
        self.assertRaises(ValueError, IPy.IP, 0xffffffff + 1, ipversion=4)
        self.assertRaises(ValueError, IPy.IP, 0xffffffffffffffffffffffffffffffff + 1, ipversion=6)

if __name__ == "__main__":
    unittest.main()




# This is a hack I use to generate my tinydns configuration
# It serves as e test for converting from Perl Net::IP to
# Python and IPy

# Further Information might be available at http://c0re.jp/c0de/IPy/
# Hacked 2001 by drt@un.bewaff.net

import sys
sys.path.append('..')

import IPy


ns = {'ns.nerxs.com': '213.221.113.70',
      'ns.dorsch.org': '195.143.234.25',
      'ns.c0re.jp': '217.6.214.130'}

print "# *** nameservers ***"
for x in ns.keys():
    print "=%s:%s" % (x, ns[x])

print "\n# *** domains ***"

fd = open('domains')

for x in fd.readlines():
    if x[0] != '#':
        if x[-1] == '\n':
            x = x[:-1]
        (domain, owner) = x.split(':')
        print "'%s:Contact for this domain is %s" % (domain, owner)
        for y in ns.keys(): 
            print ".%s::%s" % (domain, y)

fd.close()

print "\n# *** Networks ***"

fd = open('networks')
ip6map = {}
rmap = {}
nmap = {}

for x in fd.readlines():
    if x[-1] == '\n':
        x = x[:-1]
    if len(x) > 0 and x[0] != '#':
        nets = x.split(',')
        name = nets.pop(0)
        print "# Network: %s" % name
        for y in nets:
            ip = IPy.IP(y)
            print "# Address range: %s (%s), %d addresses" % (ip.strCompressed(), ip.iptype(), ip.len())
            print "=net.%s:%s" % (name, ip.net())
            print "=broadcast.%s:%s" % (name, ip.broadcast())
            
            if ip.version() == 4:
                for z in ip:
                    # TODO reverse?
                    nmap[z.int()] = name
                    rmap[z.int()] = z.strBin() + "." + name
            else:
                # IPv6
                for z in ns.keys():
                    for v in ip.reverseName():
                        print ".%s::%s" % (v, z) 
                ip6map[ip.strFullsize(0)] = name

fd.close()

print "\n# *** hosts ***"
      
fd = open('hosts')

for x in fd.readlines():
    if x[-1] == '\n':
        x = x[:-1]
    if x != '' and x[0] != '#':
        if "@Z'.".find(x[0]) >= 0:
            print x
        else:
            if "=+'".find(x[0]) >= 0:
                i = x.split(':')
                rmap[IPy.IP(i[1]).int()] = ''
                print x
            else:
                x = x[1:]
                x += '||||'
                fields = x.split('|')
                name = fields.pop(0)
                if name[0] == '.':
                    name = name[1:]
                v = fields.pop(0)
                ips = v.split(',')
                v = fields.pop(0)
                aliases = v.split(',')
                if aliases == ['']:
                    aliases = []
                admin = fields.pop()
                if admin == '':
                    admin = 'technik@c0re.23.nu'
                v = fields.pop()
                mxes = v.split(',')
                if mxes == ['']:
                    mxes = []
                for y in ips:
                    ip = IPy.IP(y) 
                    if ip.version() == 4:
                        # IPv4 is easy
                        if not nmap.has_key(ip.int()):
                            print >>sys.stderr, "*** warning: no network for %s (%s) - ignoring" % (y, name)
                            print "# no network for %s (%s)" % (y, name)
                        else:
                            print "=%s.%s:%s" % (name, nmap[ip.int()], y)
                            print "'%s.%s:Host contact is %s" % (name, nmap[ip.int()], admin)
                            rmap[ip.int()] = ''      
                            for z in aliases:
                                print "+%s:%s" % (z, ip)
                                print "'%s:Host contact is %s" % (z, admin)
                    else:
                        #IPv6 here
                        net = ip.strFullsize(0)
                        net = net[:19] + ':0000:0000:0000:0000'
                        if ip6map.has_key(net):
                            print >>sys.stderr, "*** warning: no network for %s (%s) - ignoring" % (ip, name)
                            print "# no network for %s (%s) - ignoring" % (ip, name)
                        else:  
                            print "6%s.%s:%s"; (name, ip6map[net], ip.strHex()[2:])
                            for z in aliases:
                                print "3%s:%s" % (name, ip.strHex()[2:])
                                print "'%s:Host contact is %s" % (name, admin)

fd.close()

print "\n# *** reverse lookup ***"
k = nmap.keys()
k.sort()
for x in k:
    if rmap.has_key(x) and rmap[x] != '':
      print "=%s:%s" % (rmap[x], str(IPy.IP(x)))




#!/usr/bin/env python3

'''
@Author: xxlin
@LastEditors: xxlin
@Date: 2019-04-10 13:27:58
@LastEditTime: 2019-04-10 17:46:40
'''

from typing import Any
from ..core.datatype import AttribDict

# dirmap paths
paths: AttribDict[str, str] = AttribDict()

# object to store original command line options
cmdLineOptions: AttribDict[str, Any] = AttribDict()

# object to share within function and classes command
# line options and settings
conf: AttribDict[str, Any] = AttribDict()

# object to control engine 
th: AttribDict[str, Any] = AttribDict()

#创建payloads字典对象存储payloads
payloads: AttribDict[str, Any] = AttribDict()
#URL相似度检查使用的集合
payloads.similar_urls_set = set()

#创建tasks字典对象存储tasks
tasks: AttribDict[str, list[str]] = AttribDict()

#创建进度条对象存储进度
bar: AttribDict[str, Any] = AttribDict()